@@ -44,6 +44,9 @@ struct _BoltAuth
 
   /* result */
   GError *error;
+
+  /* memory for enrollment */
+  BoltPolicy policy;
 };
 
 
@@ -57,6 +60,8 @@ enum {
   PROP_DEVICE,
   PROP_ERROR,
 
+  PROP_POLICY,
+
   PROP_LAST
 };
 
@@ -111,6 +116,10 @@ bolt_auth_get_property (GObject    *object,
       g_value_set_object (value, auth->dev);
       break;
 
+    case PROP_POLICY:
+      g_value_set_enum (value, auth->policy);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -143,6 +152,10 @@ bolt_auth_set_property (GObject      *object,
       auth->dev = g_value_dup_object (value);
       break;
 
+    case PROP_POLICY:
+      auth->policy = g_value_get_enum (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -196,6 +209,15 @@ bolt_auth_class_init (BoltAuthClass *klass)
                         G_PARAM_READWRITE |
                         G_PARAM_STATIC_NICK);
 
+  props[PROP_POLICY] =
+    g_param_spec_enum ("policy",
+                       NULL, NULL,
+                       BOLT_TYPE_POLICY,
+                       BOLT_POLICY_UNKNOWN,
+                       G_PARAM_READWRITE |
+                       G_PARAM_CONSTRUCT |
+                       G_PARAM_STATIC_STRINGS);
+
   g_object_class_install_properties (gobject_class,
                                      PROP_LAST,
                                      props);
@@ -297,6 +319,13 @@ bolt_auth_check (BoltAuth *auth,
   return TRUE;
 }
 
+BoltDevice *
+bolt_auth_get_device (BoltAuth *auth)
+{
+  g_return_val_if_fail (BOLT_IS_AUTH (auth), NULL);
+
+  return auth->dev;
+}
 
 BoltSecurity
 bolt_auth_get_level (BoltAuth *auth)
@@ -322,6 +351,14 @@ bolt_auth_get_keystate (BoltAuth *auth)
   return bolt_key_get_state (auth->key);
 }
 
+gboolean
+bolt_auth_has_key (BoltAuth *auth)
+{
+  g_return_val_if_fail (BOLT_IS_AUTH (auth), FALSE);
+
+  return auth->key != NULL;
+}
+
 gpointer
 bolt_auth_get_origin (BoltAuth *auth)
 {
@@ -330,6 +367,27 @@ bolt_auth_get_origin (BoltAuth *auth)
   return auth->origin;
 }
 
+BoltPolicy
+bolt_auth_get_policy (BoltAuth *auth)
+{
+  g_return_val_if_fail (BOLT_IS_AUTH (auth), BOLT_POLICY_UNKNOWN);
+
+  return auth->policy;
+}
+
+void
+bolt_auth_set_policy (BoltAuth  *auth,
+                      BoltPolicy policy)
+{
+  g_return_if_fail (BOLT_IS_AUTH (auth));
+
+  if (auth->policy == policy)
+    return;
+
+  auth->policy = policy;
+  g_object_notify_by_pspec (G_OBJECT (auth), props[PROP_POLICY]);
+}
+
 BoltStatus
 bolt_auth_to_status (BoltAuth *auth)
 {
