@@ -25,6 +25,7 @@
 #include "bolt-domain.h"
 #include "bolt-enums.h"
 #include "bolt-error.h"
+#include "bolt-glue.h"
 #include "bolt-io.h"
 #include "bolt-log.h"
 #include "bolt-manager.h"
@@ -509,7 +510,7 @@ bolt_device_class_init (BoltDeviceClass *klass)
 
   bolt_exported_class_set_interface_info (exported_class,
                                           BOLT_DBUS_DEVICE_INTERFACE,
-                                          "/boltd/org.freedesktop.bolt.xml");
+                                          BOLT_DBUS_GRESOURCE_PATH);
 
   bolt_exported_class_set_object_path (exported_class,
                                        BOLT_DBUS_PATH_DEVICES);
@@ -1016,7 +1017,7 @@ handle_authorize_done (GObject      *device,
 }
 
 static gboolean
-device_should_uprade_key (BoltDevice *dev)
+device_should_upgrade_key (BoltDevice *dev)
 {
   gboolean upgrade = FALSE;
   const char *reason = NULL;
@@ -1041,9 +1042,9 @@ handle_authorize (BoltExported          *object,
                   GError               **error)
 {
   g_autoptr(BoltAuth) auth = NULL;
+  g_autoptr(BoltKey) key = NULL;
   BoltDevice *dev = BOLT_DEVICE (object);
   BoltSecurity level;
-  BoltKey *key;
 
   /* In bolt_device_authorize the state is also checked, but it
    * is done already here to fail quicker and avoid accessing the
@@ -1065,19 +1066,20 @@ handle_authorize (BoltExported          *object,
     }
 
   level = bolt_domain_get_security (dev->domain);
-  key = NULL;
 
   if (level == BOLT_SECURITY_SECURE)
     {
       if (dev->key)
         key = bolt_store_get_key (dev->store, dev->uid, error);
-      else if (device_should_uprade_key (dev))
-        key = bolt_key_new ();
+      else if (device_should_upgrade_key (dev))
+        key = bolt_key_new (error);
       else
         level = BOLT_SECURITY_USER;
     }
 
-  /* only happens if the key could not be read */
+  /* happens if the key could not be read (fatal error) or if a new
+   * key could not be generated (should practically never happen).
+   * In both cases 'error' will be set. */
   if (level == BOLT_SECURITY_SECURE && key == NULL)
     return NULL;
 
@@ -1439,6 +1441,26 @@ bolt_device_get_stored (BoltDevice *dev)
   return dev->store != NULL;
 }
 
+gboolean
+bolt_device_has_iommu (BoltDevice *dev)
+{
+  g_return_val_if_fail (BOLT_IS_DEVICE (dev), FALSE);
+
+  if (dev->domain == NULL)
+    return FALSE;
+
+  return bolt_domain_has_iommu (dev->domain);
+}
+
+gboolean
+bolt_device_has_key (BoltDevice *dev)
+{
+  g_return_val_if_fail (BOLT_IS_DEVICE (dev), FALSE);
+
+  return !(dev->key == BOLT_KEY_UNKNOWN ||
+           dev->key == BOLT_KEY_MISSING);
+}
+
 const char *
 bolt_device_get_syspath (BoltDevice *dev)
 {
@@ -1544,3 +1566,29 @@ bolt_device_get_key_from_sysfs (BoltDevice *dev,
   else
     return bolt_error_propagate (error, &err);
 }
+
+gboolean
+bolt_device_load_key (BoltDevice *dev,
+                      BoltKey   **key,
+                      GError    **error)
+{
+  BoltKey *k;
+
+  g_return_val_if_fail (BOLT_IS_DEVICE (dev), FALSE);
+  g_return_val_if_fail (key != NULL, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  if (!bolt_device_has_key (dev) || dev->store == NULL)
+    {
+      *key = NULL;
+      return TRUE;
+    }
+
+  k = bolt_store_get_key (dev->store, dev->uid, error);
+
+  if (k == NULL)
+    return FALSE;
+
+  *key = k;
+  return TRUE;
+}
