@@ -98,6 +98,10 @@ guint64           bolt_device_get_conntime (BoltDevice *dev);
 
 gint64            bolt_device_get_storetime (BoltDevice *dev);
 
+gboolean          bolt_device_has_iommu (BoltDevice *dev);
+
+gboolean          bolt_device_has_key (BoltDevice *dev);
+
 gboolean          bolt_device_supports_secure_mode (BoltDevice *dev);
 
 gboolean          bolt_device_check_authflag (BoltDevice   *dev,
@@ -106,6 +110,8 @@ gboolean          bolt_device_check_authflag (BoltDevice   *dev,
 gboolean          bolt_device_get_key_from_sysfs (BoltDevice *dev,
                                                   BoltKey   **key,
                                                   GError    **error);
-
+gboolean          bolt_device_load_key (BoltDevice *dev,
+                                        BoltKey   **key,
+                                        GError    **error);
 
 G_END_DECLS
