@@ -21,6 +21,7 @@
 #include "config.h"
 
 #include "bolt-error.h"
+#include "bolt-glue.h"
 #include "bolt-log.h"
 #include "bolt-str.h"
 #include "bolt-store.h"
@@ -54,6 +55,7 @@ struct _BoltDomain
   char        *syspath;
   BoltSecurity security;
   GStrv        bootacl;
+  gboolean     iommu;
 };
 
 
@@ -70,6 +72,7 @@ enum {
   PROP_SYSPATH,
   PROP_SECURITY,
   PROP_BOOTACL,
+  PROP_IOMMU,
 
   PROP_LAST,
   PROP_EXPORTED = PROP_UID
@@ -146,6 +149,10 @@ bolt_domain_get_property (GObject    *object,
       g_value_set_boxed (value, dom->bootacl);
       break;
 
+    case PROP_IOMMU:
+      g_value_set_boolean (value, dom->iommu);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -197,6 +204,10 @@ bolt_domain_set_property (GObject      *object,
       dom->bootacl = g_value_dup_boxed (value);
       break;
 
+    case PROP_IOMMU:
+      dom->iommu = g_value_get_boolean (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -263,13 +274,21 @@ bolt_domain_class_init (BoltDomainClass *klass)
                         G_PARAM_READWRITE |
                         G_PARAM_STATIC_STRINGS);
 
+  props[PROP_IOMMU] =
+    g_param_spec_boolean ("iommu",
+                          "IOMMU", NULL,
+                          FALSE,
+                          G_PARAM_READWRITE |
+                          G_PARAM_CONSTRUCT_ONLY |
+                          G_PARAM_STATIC_STRINGS);
+
   g_object_class_install_properties (gobject_class,
                                      PROP_LAST,
                                      props);
 
   bolt_exported_class_set_interface_info (exported_class,
                                           BOLT_DBUS_DOMAIN_INTERFACE,
-                                          "/boltd/org.freedesktop.bolt.xml");
+                                          BOLT_DBUS_GRESOURCE_PATH);
 
   bolt_exported_class_set_object_path (exported_class, BOLT_DBUS_PATH_DOMAINS);
 
@@ -542,6 +561,7 @@ bolt_domain_new_for_udev (struct udev_device *udev,
   BoltSecurity security = BOLT_SECURITY_UNKNOWN;
   const char *syspath;
   const char *sysname;
+  gboolean iommu;
   gboolean ok;
   gint sort = -1;
 
@@ -565,7 +585,7 @@ bolt_domain_new_for_udev (struct udev_device *udev,
   if (g_str_has_prefix (sysname, "domain"))
     {
       const char *ptr = sysname + strlen ("domain");
-      bolt_str_parse_as_int (ptr, &sort);
+      bolt_str_parse_as_int (ptr, &sort, NULL);
     }
 
   security = bolt_sysfs_security_for_device (udev, error);
@@ -575,7 +595,19 @@ bolt_domain_new_for_udev (struct udev_device *udev,
 
   ok = bolt_sysfs_read_boot_acl (udev, &acl, &err);
   if (!ok)
-    bolt_warn_err (err, "failed to get boot_acl");
+    {
+      bolt_warn_err (err, LOG_TOPIC ("udev"),
+                     "failed to read boot_acl");
+      g_clear_error (&err);
+    }
+
+  ok = bolt_sysfs_read_iommu (udev, &iommu, &err);
+  if (!ok)
+    {
+      bolt_warn_err (err, LOG_TOPIC ("udev"),
+                     "failed to read iommu");
+      g_clear_error (&err);
+    }
 
   dom = g_object_new (BOLT_TYPE_DOMAIN,
                       "uid", uid,
@@ -583,6 +615,7 @@ bolt_domain_new_for_udev (struct udev_device *udev,
                       "syspath", syspath,
                       "security", security,
                       "bootacl", acl,
+                      "iommu", iommu,
                       NULL);
 
   return dom;
@@ -652,6 +685,14 @@ bolt_domain_is_connected (BoltDomain *domain)
   return domain->syspath != NULL;
 }
 
+gboolean
+bolt_domain_has_iommu (BoltDomain *domain)
+{
+  g_return_val_if_fail (BOLT_IS_DOMAIN (domain), FALSE);
+
+  return domain->iommu;
+}
+
 void
 bolt_domain_export (BoltDomain      *domain,
                     GDBusConnection *bus)
@@ -687,6 +728,7 @@ bolt_domain_connected (BoltDomain         *domain,
   BoltSecurity security;
   const char *syspath;
   const char *id;
+  gboolean iommu;
   gboolean ok;
 
   g_return_if_fail (BOLT_IS_DOMAIN (domain));
@@ -714,15 +756,25 @@ bolt_domain_connected (BoltDomain         *domain,
       g_clear_error (&err);
     }
 
+  ok = bolt_sysfs_read_iommu (dev, &iommu, &err);
+  if (!ok)
+    {
+      bolt_warn_err (err, LOG_TOPIC ("udev"),
+                     "failed to read iommu");
+      g_clear_error (&err);
+    }
+
   g_object_freeze_notify (G_OBJECT (domain));
 
   domain->id = g_strdup (id);
   domain->syspath = g_strdup (syspath);
   domain->security = security;
+  domain->iommu = iommu;
 
   g_object_notify_by_pspec (G_OBJECT (domain), props[PROP_ID]);
   g_object_notify_by_pspec (G_OBJECT (domain), props[PROP_SYSPATH]);
   g_object_notify_by_pspec (G_OBJECT (domain), props[PROP_SECURITY]);
+  g_object_notify_by_pspec (G_OBJECT (domain), props[PROP_IOMMU]);
 
   ok = bolt_sysfs_read_boot_acl (dev, &acl, &err);
   if (!ok)
