@@ -336,7 +336,7 @@ static void
 bolt_manager_init (BoltManager *mgr)
 {
   mgr->devices = g_ptr_array_new_with_free_func (g_object_unref);
-  mgr->store = bolt_store_new (g_getenv ("BOLT_DBPATH") ? : BOLT_DBDIR);
+  mgr->store = bolt_store_new (bolt_get_store_path ());
 
   mgr->probing_roots = g_ptr_array_new_with_free_func (g_free);
   mgr->probing_tsettle = PROBING_SETTLE_TIME_MS; /* milliseconds */
@@ -410,7 +410,7 @@ bolt_manager_class_init (BoltManagerClass *klass)
 
   bolt_exported_class_set_interface_info (exported_class,
                                           BOLT_DBUS_INTERFACE,
-                                          "/boltd/org.freedesktop.bolt.xml");
+                                          BOLT_DBUS_GRESOURCE_PATH);
 
   bolt_exported_class_export_properties (exported_class,
                                          PROP_EXPORTED,
@@ -715,14 +715,16 @@ manager_domain_ensure (BoltManager        *mgr,
                        struct udev_device *dev)
 {
   g_autoptr(GError) err = NULL;
-  BoltSecurity security;
+  BoltSecurity level;
   BoltDomain *domain = NULL;
   GDBusConnection *bus;
   struct udev_device *host;
   struct udev_device *dom;
+  const char *security;
   const char *syspath;
   const char *op;
   const char *uid;
+  gboolean iommu;
   gboolean ok;
 
   /* check if we already know a domain that is the parent
@@ -769,12 +771,14 @@ manager_domain_ensure (BoltManager        *mgr,
       return NULL;
     }
 
-  security = bolt_domain_get_security (domain);
+  level = bolt_domain_get_security (domain);
+  iommu = bolt_domain_has_iommu (domain);
+  security = bolt_security_for_display (level, iommu);
 
   bolt_msg (LOG_DOM (domain), "newly connected [%s] (%s)",
-            bolt_security_to_string (security), syspath);
+            security, syspath);
 
-  manager_maybe_set_security (mgr, security);
+  manager_maybe_set_security (mgr, level);
 
   manager_register_domain (mgr, domain);
 
@@ -1130,9 +1134,9 @@ bolt_manager_label_device (BoltManager *mgr,
 
 /* device authorization */
 static void
-authorize_device_finish (GObject      *source,
-                         GAsyncResult *res,
-                         gpointer      user_data)
+auto_auth_done (GObject      *source,
+                GAsyncResult *res,
+                gpointer      user_data)
 {
   g_autoptr(GError) err = NULL;
   BoltDevice *dev = BOLT_DEVICE (source);
@@ -1142,110 +1146,284 @@ authorize_device_finish (GObject      *source,
   ok = bolt_auth_check (auth, &err);
 
   if (!ok)
-    bolt_warn_err (err, LOG_DEV (dev), "authorization failed");
+    bolt_warn_err (err, LOG_DEV (dev), LOG_TOPIC ("auto-auth"),
+                   "authorization failed");
   else
-    bolt_msg (LOG_DEV (dev), "authorized");
+    bolt_msg (LOG_DEV (dev), LOG_TOPIC ("auto-auth"),
+              "authorization successful");
 }
 
 static void
-maybe_authorize_device (BoltManager *mgr,
+manager_auto_authorize (BoltManager *mgr,
                         BoltDevice  *dev)
 {
   g_autoptr(BoltAuth) auth = NULL;
-  BoltStatus status = bolt_device_get_status (dev);
-  BoltPolicy policy = bolt_device_get_policy (dev);
-  const char *uid = bolt_device_get_uid (dev);
-  BoltKey *key = NULL;
+  g_autoptr(BoltKey) key = NULL;
+  g_autofree char *amstr = NULL;
+  BoltStatus status;
+  BoltPolicy policy;
+  gboolean authmode;
+  gboolean authorize;
+  gboolean iommu;
   BoltSecurity level;
-  gboolean stored;
+  gboolean ok;
 
-  bolt_info (LOG_DEV (dev), "checking possible authorization: %s (%x)",
-             bolt_policy_to_string (policy), status);
-
-  if (bolt_auth_mode_is_disabled (mgr->authmode))
-    {
-      bolt_info (LOG_DEV (dev), "authorization is globally disabled.");
-      return;
-    }
+  status = bolt_device_get_status (dev);
+  policy = bolt_device_get_policy (dev);
 
+  /* more of a sanity check, because we should only end up
+  * here if the device is not yet authorized and stored */
   if (bolt_status_is_authorized (status) ||
-      policy != BOLT_POLICY_AUTO)
+      !bolt_device_get_stored (dev))
     return;
 
-  stored = bolt_device_get_stored (dev);
-  /* sanity check, because we already checked the policy */
-  g_return_if_fail (stored);
+  /* The default is not to authorize anything */
+  authorize = FALSE;
 
+  /* The following sections are security critical and thus
+   * intentionally verbose and explicit  */
+
+  /* 1) global auth mode and policy check */
+  authmode = bolt_auth_mode_is_enabled (mgr->authmode);
+  amstr = bolt_auth_mode_to_string (authmode);
+
+  iommu = bolt_device_has_iommu (dev);
   level = bolt_device_get_security (dev);
-  if (level == BOLT_SECURITY_SECURE &&
-      bolt_device_get_keystate (dev) != BOLT_KEY_MISSING)
+
+  if (authmode)
+    {
+      if (policy == BOLT_POLICY_AUTO)
+        authorize = TRUE;
+      else if (policy == BOLT_POLICY_IOMMU && iommu)
+        authorize = TRUE;
+    }
+
+  bolt_msg (LOG_DEV (dev), LOG_TOPIC ("auto-auth"),
+            "authmode: %s, policy: %s, iommu: %s -> %s",
+            amstr, bolt_policy_to_string (policy),
+            bolt_yesno (iommu), bolt_okfail (authorize));
+
+  if (!authorize)
+    return;
+
+  /* 2) security level and key check: if we are in SECURE
+   *    mode but don't have a key, we DON'T authorize */
+  if (level == BOLT_SECURITY_SECURE)
     {
       g_autoptr(GError) err = NULL;
-      key = bolt_store_get_key (mgr->store, uid, &err);
-      if (key == NULL)
+
+      ok = bolt_device_load_key (dev, &key, &err);
+      if (!ok)
         bolt_warn_err (err, LOG_DEV (dev), "could not load key");
+
+      authorize = (key != NULL);
     }
 
-  if (level == BOLT_SECURITY_SECURE && key == NULL)
-    {
-      bolt_msg (LOG_DEV (dev), "have no key, not authorizing (secure mode)");
-      return;
-    }
+  bolt_msg (LOG_DEV (dev), LOG_TOPIC ("auto-auth"),
+            "security: %s mode, key: %s -> %s",
+            bolt_security_for_display (level, iommu),
+            bolt_yesno (key), bolt_okfail (authorize));
+
+  if (!authorize)
+    return;
 
   auth = bolt_auth_new (mgr, level, key);
-  bolt_device_authorize_idle (dev, auth, authorize_device_finish, mgr);
+  bolt_device_authorize_idle (dev, auth, auto_auth_done, mgr);
 }
 
 static void
-manager_maybe_auto_import_device (BoltManager *mgr,
-                                  BoltDevice  *dev)
+manager_maybe_import (BoltManager *mgr,
+                      BoltDevice  *dev)
 {
   g_autoptr(GError) err = NULL;
   g_autoptr(BoltKey) key = NULL;
-  gboolean boot;
+  BoltSecurity level;
+  BoltPolicy policy;
+  const char *secstr;
+  const char *polstr;
+  gboolean sl0, sl1;
+  gboolean boot, pcie;
+  gboolean import;
+  gboolean iommu;
   gboolean ok;
 
-  if (!bolt_device_is_authorized (dev))
-    return;
-
   if (bolt_device_get_device_type (dev) == BOLT_DEVICE_HOST)
     return;
 
+  g_return_if_fail (!bolt_device_get_stored (dev));
+  g_return_if_fail (bolt_device_is_authorized (dev));
+
+  /* This function is intentionally more verbose then it
+   * could be, but since it is security critical, it is
+   * better to be clear than concise */
+
+  level = bolt_device_get_security (dev);
+  iommu = bolt_device_has_iommu (dev);
   boot = bolt_device_check_authflag (dev, BOLT_AUTH_BOOT);
 
-  if (!boot)
-    {
-      bolt_msg (LOG_DEV (dev), LOG_TOPIC ("auto-import"),
-                "boot flag missing, not auto-importing");
-      return;
-    }
+  pcie = bolt_security_allows_pcie (level);
+  sl0 = level == BOLT_SECURITY_NONE;
+  sl1 = level == BOLT_SECURITY_USER;
 
-  ok = bolt_device_get_key_from_sysfs (dev, &key, &err);
-  if (!ok)
-    {
-      bolt_warn_err (err, LOG_DEV (dev), LOG_TOPIC ("auto-import"),
-                     "failed to read key from sysfs");
-      return;
-    }
+  /* Check if we want to import that device at all,
+   * the fundamental rule is: if it was authorized
+   * by the firmware and we have pcie tunnels we want
+   * to import it so we have a record of it */
+  import = pcie && (boot || sl0);
 
-  if (key == NULL && mgr->security == BOLT_SECURITY_SECURE)
-    {
-      bolt_msg (LOG_DEV (dev), LOG_TOPIC ("auto-import"),
-                "secure mode enabled, no key, not auto-importing");
-      return;
-    }
+  if (import && !iommu && sl1)
+    policy = BOLT_POLICY_AUTO;
+  else
+    policy = BOLT_POLICY_IOMMU;
 
-  bolt_msg (LOG_DEV (dev), LOG_TOPIC ("auto-import"),
-            "new authorized device (boot: %s, key: %s), importing",
-            bolt_yesno (boot), bolt_yesno (key != NULL));
+  secstr = bolt_security_for_display (level, iommu);
+  polstr = bolt_policy_to_string (policy);
 
-  ok = bolt_store_put_device (mgr->store, dev, BOLT_POLICY_AUTO, key, &err);
+  bolt_msg (LOG_DEV (dev), LOG_TOPIC ("import"),
+            "%s mode, boot: %s, key: %s -> %s",
+            secstr, bolt_yesno (boot), bolt_yesno (key),
+            (import ? polstr : "no import"));
+
+  if (!import)
+    return;
+
+  ok = bolt_store_put_device (mgr->store,
+                              dev,
+                              policy,
+                              key, &
+                              err);
 
   if (!ok)
-    bolt_warn_err (err, LOG_DEV (dev), LOG_TOPIC ("auto-import"),
+    bolt_warn_err (err, LOG_DEV (dev), LOG_TOPIC ("import"),
                    "failed to store device");
 }
 
+static void
+auto_enroll_done (GObject      *device,
+                  GAsyncResult *res,
+                  gpointer      user_data)
+{
+  g_autoptr(GError) err = NULL;
+  BoltDevice *dev = BOLT_DEVICE (device);
+  BoltAuth *auth = BOLT_AUTH (res);
+  BoltManager *mgr;
+  BoltPolicy policy;
+  BoltKey *key;
+  gboolean ok;
+
+  mgr = BOLT_MANAGER (bolt_auth_get_origin (auth));
+  ok = bolt_auth_check (auth, &err);
+
+  if (!ok)
+    {
+      bolt_warn_err (err, LOG_DEV (dev), LOG_TOPIC ("auto-enroll"),
+                     "failed to authorize the new device");
+      return;
+    }
+
+  key = bolt_auth_get_key (auth);
+  policy = bolt_auth_get_policy (auth);
+
+  ok = bolt_store_put_device (mgr->store, dev, policy, key, &err);
+
+  if (ok)
+    bolt_msg (LOG_DEV (dev), LOG_TOPIC ("auto-enroll"), "done");
+  else
+    bolt_warn_err (err, LOG_DEV (dev), LOG_TOPIC ("auto-enroll"),
+                   "failed to store the device");
+}
+
+static BoltAuth *
+manager_enroll_device_prepare (BoltManager *mgr,
+                               BoltDevice  *dev,
+                               GError     **error)
+{
+  g_autoptr(BoltDevice) parent = NULL;
+  g_autoptr(BoltKey) key = NULL;
+  BoltSecurity level;
+  BoltAuth *auth;
+
+  /* are we even allowed to enroll new devices right now? */
+  if (bolt_auth_mode_is_disabled (mgr->authmode))
+    {
+      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_ACCESS_DENIED,
+                   "authorization of new devices is disabled");
+      return NULL;
+    }
+
+  /* if the parent is not authorized and we try to authorize,
+   * we will fail and log a warning, so lets check, fail early
+   * and avoid the warning */
+  parent = bolt_manager_get_parent (mgr, dev);
+  if (!bolt_device_is_authorized (parent))
+    {
+      g_set_error (error, BOLT_ERROR, BOLT_ERROR_AUTHCHAIN,
+                   "parent not authorized, deferring");
+      return NULL;
+    }
+
+  /* determine the maximum security level we can use, i.e.
+   * minimum level that the device *and* host support */
+  if (bolt_device_supports_secure_mode (dev))
+    level = bolt_device_get_security (dev);
+  else
+    level = BOLT_SECURITY_USER;
+
+  /* yay, we are in SECURE mode, we need to create a key */
+  if (level == BOLT_SECURITY_SECURE)
+    {
+      key = bolt_key_new (error);
+      if (key == NULL)
+        return NULL;
+    }
+
+  auth = bolt_auth_new (mgr, level, key);
+
+  /* bolt_auth_new is a wrapper around g_object_new and
+   * therefore cannot realistically fail. This assertion
+   * convinces clang's static analysis of that fact. */
+  g_assert (auth != NULL);
+
+  return auth;
+}
+
+static void
+manager_auto_enroll (BoltManager *mgr,
+                     BoltDevice  *dev)
+{
+  g_autoptr(BoltAuth) auth = NULL;
+  g_autoptr(GError) err = NULL;
+  BoltStatus status;
+  gboolean have_key;
+
+  g_return_if_fail (!bolt_device_get_stored (dev));
+
+  /* sanity check for the state */
+  status = bolt_device_get_status (dev);
+  if (!bolt_status_is_pending (status))
+    return;
+
+  /* sanity check for iommu */
+  if (!bolt_device_has_iommu (dev))
+    return;
+
+  auth = manager_enroll_device_prepare (mgr, dev, &err);
+
+  if (!auth)
+    {
+      bolt_msg (LOG_DEV (dev), LOG_TOPIC ("auto-enroll"),
+                "no, pre-check failed: %s", err->message);
+      return;
+    }
+
+  have_key = bolt_auth_has_key (auth);
+  bolt_msg (LOG_DEV (dev), LOG_TOPIC ("auto-enroll"),
+            "yes, key: %s", bolt_yesno (have_key));
+
+  bolt_auth_set_policy (auth, BOLT_POLICY_IOMMU);
+  bolt_device_authorize (dev, auth, auto_enroll_done, mgr);
+}
+
 /* udev callbacks */
 static void
 handle_uevent_udev (BoltUdev           *udev,
@@ -1460,7 +1638,10 @@ handle_udev_device_added (BoltManager        *mgr,
 
   bolt_manager_label_device (mgr, dev);
 
-  manager_maybe_auto_import_device (mgr, dev);
+  if (bolt_status_is_authorized (status))
+    manager_maybe_import (mgr, dev);
+  else if (bolt_domain_has_iommu (domain))
+    manager_auto_enroll (mgr, dev);
 
   /* if we have a valid dbus connection */
   bus = bolt_exported_get_connection (BOLT_EXPORTED (mgr));
@@ -1563,7 +1744,7 @@ handle_udev_device_attached (BoltManager        *mgr,
       bolt_warn (LOG_DEV (dev), "could not find parent");
     }
 
-  maybe_authorize_device (mgr, dev);
+  manager_auto_authorize (mgr, dev);
 }
 
 static void
@@ -1764,7 +1945,10 @@ handle_device_status_changed (BoltDevice  *dev,
   for (guint i = 0; i < children->len; i++)
     {
       BoltDevice *child = g_ptr_array_index (children, i);
-      maybe_authorize_device (mgr, child);
+      if (bolt_device_get_stored (child))
+        manager_auto_authorize (mgr, child);
+      else if (bolt_device_has_iommu (child))
+        manager_auto_enroll (mgr, child);
     }
 }
 
@@ -2207,36 +2391,22 @@ enroll_device_done (GObject      *device,
                     GAsyncResult *res,
                     gpointer      user_data)
 {
-  BoltAuth *auth;
-  BoltDevice *dev;
-  BoltManager *mgr;
-  GDBusMethodInvocation *inv;
+  GDBusMethodInvocation *inv = user_data;
+  BoltDevice *dev = BOLT_DEVICE (device);
+  BoltAuth *auth = BOLT_AUTH (res);
   GError *error = NULL;
+  BoltManager *mgr;
   const char *opath;
   gboolean ok;
 
-  inv = user_data;
-  dev = BOLT_DEVICE (device);
-  auth = BOLT_AUTH (res);
   mgr = BOLT_MANAGER (bolt_auth_get_origin (auth));
   ok = bolt_auth_check (auth, &error);
 
   if (ok)
     {
-      GVariant *params;
-      const char *str;
-      BoltPolicy policy;
-
-      params = g_dbus_method_invocation_get_parameters (inv);
-      g_variant_get_child (params, 1, "&s", &str);
-
-      policy = bolt_enum_from_string (BOLT_TYPE_POLICY, str, NULL);
-      if (policy == BOLT_POLICY_DEFAULT)
-        policy = mgr->policy;
-
       ok = bolt_store_put_device (mgr->store,
                                   dev,
-                                  policy,
+                                  bolt_auth_get_policy (auth),
                                   bolt_auth_get_key (auth),
                                   &error);
     }
@@ -2297,10 +2467,8 @@ handle_enroll_device (BoltExported          *obj,
 {
   g_autoptr(BoltDevice) dev = NULL;
   g_autoptr(BoltAuth) auth = NULL;
-  g_autoptr(BoltKey) key = NULL;
   BoltManager *mgr;
   const char *uid;
-  BoltSecurity level;
   BoltPolicy pol;
   const char *policy;
 
@@ -2323,8 +2491,14 @@ handle_enroll_device (BoltExported          *obj,
     }
   else if (pol == BOLT_POLICY_DEFAULT)
     {
-      pol = mgr->policy;
-      bolt_info (LOG_DEV (dev), "got 'default' policy, adjusting");
+      if (bolt_device_has_iommu (dev))
+        pol = BOLT_POLICY_IOMMU;
+      else
+        pol = mgr->policy;
+
+      bolt_info (LOG_DEV (dev), LOG_TOPIC ("enroll"),
+                 "got 'default' policy, adjusted to: '%s'",
+                 bolt_policy_to_string (pol));
     }
 
   if (bolt_device_get_stored (dev))
@@ -2339,25 +2513,13 @@ handle_enroll_device (BoltExported          *obj,
   if (bolt_device_is_authorized (dev))
     return enroll_device_store_authorized (mgr, dev, pol, error);
 
-  if (bolt_auth_mode_is_disabled (mgr->authmode))
-    {
-      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_ACCESS_DENIED,
-                   "authorization of new devices is disabled");
-      return NULL;
-    }
+  auth = manager_enroll_device_prepare (mgr, dev, error);
 
-  if (bolt_device_supports_secure_mode (dev))
-    level = bolt_device_get_security (dev);
-  else
-    level = BOLT_SECURITY_USER;
+  if (auth == NULL)
+    return NULL;
 
-  key = NULL;
-  if (level == BOLT_SECURITY_SECURE)
-    key = bolt_key_new ();
-
-  auth = bolt_auth_new (mgr, level, key);
+  bolt_auth_set_policy (auth, pol);
   bolt_device_authorize (dev, auth, enroll_device_done, inv);
-
   return NULL;
 
 }
