@@ -23,6 +23,7 @@
 #include "bolt-device.h"
 #include "bolt-error.h"
 #include "bolt-names.h"
+#include "bolt-str.h"
 
 #include <gio/gio.h>
 
@@ -76,18 +77,6 @@ G_DEFINE_TYPE (BoltClient,
                BOLT_TYPE_PROXY);
 
 
-static void
-bolt_client_get_property (GObject    *object,
-                          guint       prop_id,
-                          GValue     *value,
-                          GParamSpec *pspec)
-{
-  if (bolt_proxy_get_dbus_property (object, pspec, value))
-    return;
-
-  G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-}
-
 static const BoltProxySignal *
 bolt_client_get_dbus_signals (guint *n)
 {
@@ -110,54 +99,56 @@ bolt_client_class_init (BoltClientClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   BoltProxyClass *proxy_class = BOLT_PROXY_CLASS (klass);
 
-  gobject_class->get_property = bolt_client_get_property;
+  gobject_class->get_property = bolt_proxy_property_getter;
+  gobject_class->set_property = bolt_proxy_property_setter;
 
   proxy_class->get_dbus_signals = bolt_client_get_dbus_signals;
 
   props[PROP_VERSION]
-    = g_param_spec_uint ("version",
-                         "Version", NULL,
+    = g_param_spec_uint ("version", "Version",
+                         "D-Bus API revision.",
                          0, G_MAXUINT, 0,
                          G_PARAM_READABLE |
-                         G_PARAM_STATIC_NAME);
+                         G_PARAM_STATIC_STRINGS);
 
   props[PROP_PROBING]
-    = g_param_spec_boolean ("probing",
-                            "Probing", NULL,
+    = g_param_spec_boolean ("probing", "Probing",
+                            "Is the daemon processing new devices?",
                             FALSE,
                             G_PARAM_READABLE |
-                            G_PARAM_STATIC_NAME);
+                            G_PARAM_STATIC_STRINGS);
 
   props[PROP_POLICY]
-    = g_param_spec_enum ("default-policy",
-                         "DefaultPolicy", NULL,
+    = g_param_spec_enum ("default-policy", "DefaultPolicy",
+                         "Policy to use if none was specified.",
                          BOLT_TYPE_POLICY,
                          BOLT_POLICY_UNKNOWN,
                          G_PARAM_READABLE |
-                         G_PARAM_STATIC_NAME);
+                         G_PARAM_STATIC_STRINGS);
 
   props[PROP_SECURITY]
-    = g_param_spec_enum ("security-level",
-                         "SecurityLevel", NULL,
+    = g_param_spec_enum ("security-level", "SecurityLevel",
+                         "The global security level.",
                          BOLT_TYPE_SECURITY,
                          BOLT_SECURITY_UNKNOWN,
                          G_PARAM_READABLE |
-                         G_PARAM_STATIC_NAME);
+                         G_PARAM_STATIC_STRINGS);
 
   props[PROP_AUTHMODE] =
-    g_param_spec_flags ("auth-mode", "AuthMode", NULL,
+    g_param_spec_flags ("auth-mode", "AuthMode",
+                        "Current authentication mode.",
                         BOLT_TYPE_AUTH_MODE,
                         BOLT_AUTH_ENABLED,
-                        G_PARAM_READABLE |
+                        G_PARAM_READWRITE |
                         G_PARAM_STATIC_STRINGS);
 
   props[PROP_POWERSTATE] =
-    g_param_spec_enum ("power-state",
-                       "PowerState", NULL,
+    g_param_spec_enum ("power-state", "PowerState",
+                       "Current force power state.",
                        BOLT_TYPE_POWER_STATE,
                        BOLT_FORCE_POWER_UNSET,
                        G_PARAM_READABLE |
-                       G_PARAM_STATIC_NAME);
+                       G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (gobject_class,
                                      PROP_LAST,
@@ -297,7 +288,7 @@ got_the_client (GObject      *source,
                 GAsyncResult *res,
                 gpointer      user_data)
 {
-  g_autoptr(GError) error = NULL;
+  GError *error = NULL;
   GTask *task = user_data;
   GObject *obj;
 
@@ -305,6 +296,7 @@ got_the_client (GObject      *source,
 
   if (obj == NULL)
     {
+      /* error ownership gets transferred to the task */
       g_task_return_error (task, error);
       return;
     }
@@ -318,7 +310,7 @@ got_the_bus (GObject      *source,
              GAsyncResult *res,
              gpointer      user_data)
 {
-  g_autoptr(GError) error = NULL;
+  GError *error = NULL;
   GTask *task = user_data;
   GCancellable *cancellable;
   GDBusConnection *bus;
@@ -327,6 +319,7 @@ got_the_bus (GObject      *source,
   if (bus == NULL)
     {
       g_prefix_error (&error, "could not connect to D-Bus: ");
+      /* error ownership gets transferred to the task */
       g_task_return_error (task, error);
       return;
     }
@@ -413,6 +406,31 @@ bolt_client_list_domains (BoltClient   *client,
   return g_steal_pointer (&domains);
 }
 
+BoltDomain *
+bolt_client_find_domain (BoltClient *client,
+                         const char *name,
+                         GError    **error)
+{
+  g_autoptr(GPtrArray) domains = NULL;
+
+  domains = bolt_client_list_domains (client, NULL, error);
+  if (domains == NULL)
+    return NULL;
+
+  for (guint i = 0; i < domains->len; i++)
+    {
+      BoltDomain *dom = g_ptr_array_index (domains, i);
+
+      if (bolt_streq (name, bolt_domain_get_uid (dom)) ||
+          bolt_streq (name, bolt_domain_get_id (dom)))
+        return g_object_ref (dom);
+    }
+
+  g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
+               "could not find domain matching '%s'", name);
+  return NULL;
+}
+
 GPtrArray *
 bolt_client_list_devices (BoltClient   *client,
                           GCancellable *cancel,
@@ -457,6 +475,40 @@ bolt_client_list_devices (BoltClient   *client,
   return g_steal_pointer (&devices);
 }
 
+GPtrArray *
+bolt_client_list_parents (BoltClient   *client,
+                          BoltDevice   *device,
+                          GCancellable *cancellable,
+                          GError      **error)
+{
+  g_autoptr(BoltDevice) dev = NULL;
+  g_autoptr(GPtrArray) uuids = NULL;
+  const char *parent;
+
+  uuids = g_ptr_array_new_full (16, g_object_unref);
+  dev = g_object_ref (device);
+
+  while ((parent = bolt_device_get_parent (dev)) != NULL)
+    {
+      g_autofree char *up = g_strdup (parent);
+
+      g_clear_object (&dev); /* 'parent' can be invalid */
+      parent = NULL;
+
+      dev = bolt_client_get_device (client,
+                                    up,
+                                    NULL,
+                                    error);
+
+      if (dev == NULL)
+        return NULL;
+
+      g_ptr_array_add (uuids, g_object_ref (dev));
+    }
+
+  return g_steal_pointer (&uuids);
+}
+
 BoltDevice *
 bolt_client_get_device (BoltClient   *client,
                         const char   *uid,
@@ -498,6 +550,31 @@ bolt_client_get_device (BoltClient   *client,
   return dev;
 }
 
+BoltDevice *
+bolt_client_find_device (BoltClient *client,
+                         const char *name,
+                         GError    **error)
+{
+  g_autoptr(GPtrArray) devices = NULL;
+
+  devices = bolt_client_list_devices (client, NULL, error);
+  if (devices == NULL)
+    return NULL;
+
+  for (guint i = 0; i < devices->len; i++)
+    {
+      BoltDevice *dev = g_ptr_array_index (devices, i);
+
+      if (bolt_streq (name, bolt_device_get_uid (dev)) ||
+          bolt_streq (name, bolt_device_get_name (dev)))
+        return g_object_ref (dev);
+    }
+
+  g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
+               "could not find device matching '%s'", name);
+  return NULL;
+}
+
 BoltDevice *
 bolt_client_enroll_device (BoltClient  *client,
                            const char  *uid,
@@ -602,7 +679,7 @@ bolt_client_enroll_device_finish (BoltClient   *client,
                                   GError      **error)
 {
   g_autoptr(GError) err = NULL;
-  GVariant *val = NULL;
+  g_autoptr(GVariant) val = NULL;
 
   g_return_val_if_fail (BOLT_IS_CLIENT (client), FALSE);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
@@ -621,6 +698,360 @@ bolt_client_enroll_device_finish (BoltClient   *client,
   return TRUE;
 }
 
+typedef struct OpData
+{
+  const char *iface;  /* Manager or Device */
+  const char *method; /* Enroll or Authorize */
+  char       *path;   /* object path */
+  GVariant   *params; /* parameters */
+
+} OpData;
+
+static OpData *
+op_data_new_enroll (const char *uid,
+                    const char *policy,
+                    const char *flags)
+{
+  GVariant *params;
+  OpData *op;
+
+  params = g_variant_new ("(sss)", uid, policy, flags);
+
+  op = g_slice_new (OpData);
+  op->iface = BOLT_DBUS_INTERFACE;
+  op->method = "EnrollDevice";
+  op->params = g_variant_ref_sink (params);
+  op->path = g_strdup (BOLT_DBUS_PATH);
+
+  return op;
+}
+
+
+static OpData *
+op_data_new_authorize (const char *uid,
+                       const char *flags)
+{
+  OpData *op = NULL;
+  GVariant *params;
+  char *path;
+
+  path = bolt_gen_object_path (BOLT_DBUS_PATH_DEVICES, uid);
+  params = g_variant_new ("(s)", flags);
+
+  op = g_slice_new (OpData);
+
+  op->iface = BOLT_DBUS_DEVICE_INTERFACE;
+  op->method = "Authorize";
+  op->params = g_variant_ref_sink (params);
+  op->path = path; /* takes ownership */
+
+  return op;
+}
+
+static void
+op_data_free (OpData *op)
+{
+  g_clear_pointer (&op->params, g_variant_unref);
+  g_clear_pointer (&op->path, g_free);
+  g_slice_free (OpData, op);
+}
+
+static void
+op_queue_free (GQueue *queue)
+{
+  g_queue_free_full (queue, (GDestroyNotify) op_data_free);
+}
+
+static void    allop_one_done (GObject      *source_object,
+                               GAsyncResult *res,
+                               gpointer      user_data);
+
+static gboolean
+allop_continue (BoltClient *client, GTask *task, GQueue *ops)
+{
+  GDBusConnection *bus;
+  GCancellable *cancel;
+  OpData *op;
+
+  cancel = g_task_get_cancellable (task);
+  bus = g_dbus_proxy_get_connection (G_DBUS_PROXY (client));
+
+  op = g_queue_pop_head (ops);
+
+  if (op == NULL)
+    return TRUE;
+
+  g_dbus_connection_call (bus,
+                          BOLT_DBUS_NAME,
+                          op->path,
+                          op->iface,
+                          op->method,
+                          op->params,
+                          NULL,
+                          G_DBUS_CALL_FLAGS_NONE,
+                          -1,
+                          cancel,
+                          allop_one_done,
+                          task);
+
+  op_data_free (op);
+
+  return FALSE;
+}
+
+static void
+allop_one_done (GObject      *source_object,
+                GAsyncResult *res,
+                gpointer      user_data)
+{
+  g_autoptr(GVariant) val = NULL;
+  BoltClient *client;
+  GDBusConnection *bus;
+  gboolean done;
+  GError *err = NULL;
+  GQueue *ops;
+  GTask *task;
+
+  task = G_TASK (user_data);
+
+  ops = g_task_get_task_data (task);
+  client = g_task_get_source_object (task);
+  bus = g_dbus_proxy_get_connection (G_DBUS_PROXY (client));
+
+  val = g_dbus_connection_call_finish (bus, res, &err);
+
+  if (val == NULL)
+    {
+      g_task_return_error (task, err); /* takes ownership */
+      g_object_unref (task);
+      /* we are done (albeit with an error) */
+      return;
+    }
+
+  done = allop_continue (client, task, ops);
+
+  if (done)
+    {
+      /* we are done */
+      g_task_return_boolean (task, TRUE);
+      g_object_unref (task);
+    }
+}
+
+void
+bolt_client_enroll_all_async (BoltClient         *client,
+                              GPtrArray          *uuids,
+                              BoltPolicy          policy,
+                              BoltAuthCtrl        flags,
+                              GCancellable       *cancellable,
+                              GAsyncReadyCallback callback,
+                              gpointer            user_data)
+{
+  g_autofree char *fstr = NULL;
+  GError *err = NULL;
+  const char *pstr;
+  GQueue *ops;
+  GTask *task;
+
+  g_return_if_fail (BOLT_IS_CLIENT (client));
+  g_return_if_fail (uuids != NULL && uuids->len > 0);
+  g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
+  g_return_if_fail (callback != NULL);
+
+  pstr = bolt_enum_to_string (BOLT_TYPE_POLICY, policy, &err);
+  if (pstr == NULL)
+    {
+      g_task_report_error (client, callback, user_data, NULL, err);
+      return;
+    }
+
+  fstr = bolt_flags_to_string (BOLT_TYPE_AUTH_CTRL, flags, &err);
+  if (fstr == NULL)
+    {
+      g_task_report_error (client, callback, user_data, NULL, err);
+      return;
+    }
+
+  task = g_task_new (client, cancellable, callback, user_data);
+  g_task_set_return_on_cancel (task, TRUE);
+
+  ops = g_queue_new ();
+  g_task_set_task_data (task, ops, (GDestroyNotify) op_queue_free);
+
+  for (guint i = 0; i < uuids->len; i++)
+    {
+      const char *uid = g_ptr_array_index (uuids, i);
+      OpData *op;
+
+      op = op_data_new_enroll (uid, pstr, fstr);
+
+      g_queue_push_tail (ops, op);
+    }
+
+  allop_continue (client, task, ops);
+}
+
+gboolean
+bolt_client_enroll_all_finish (BoltClient   *client,
+                               GAsyncResult *res,
+                               GError      **error)
+{
+  g_autoptr(GError) err = NULL;
+  gboolean ok;
+
+  g_return_val_if_fail (BOLT_IS_CLIENT (client), FALSE);
+  g_return_val_if_fail (g_task_is_valid (res, client), FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  ok = g_task_propagate_boolean (G_TASK (res), &err);
+
+  if (!ok)
+    bolt_error_propagate_stripped (error, &err);
+
+  return ok;
+}
+
+void
+bolt_client_authorize_all_async (BoltClient         *client,
+                                 GPtrArray          *uuids,
+                                 BoltAuthCtrl        flags,
+                                 GCancellable       *cancellable,
+                                 GAsyncReadyCallback callback,
+                                 gpointer            user_data)
+{
+  g_autofree char *fstr = NULL;
+  GError *err = NULL;
+  GQueue *ops;
+  GTask *task;
+
+  g_return_if_fail (BOLT_IS_CLIENT (client));
+  g_return_if_fail (uuids != NULL && uuids->len > 0);
+  g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
+  g_return_if_fail (callback != NULL);
+
+  fstr = bolt_flags_to_string (BOLT_TYPE_AUTH_CTRL, flags, &err);
+  if (fstr == NULL)
+    {
+      g_task_report_error (client, callback, user_data, NULL, err);
+      return;
+    }
+
+  task = g_task_new (client, cancellable, callback, user_data);
+  g_task_set_return_on_cancel (task, TRUE);
+
+  ops = g_queue_new ();
+  g_task_set_task_data (task, ops, (GDestroyNotify) op_queue_free);
+
+  for (guint i = 0; i < uuids->len; i++)
+    {
+      const char *uid = g_ptr_array_index (uuids, i);
+      OpData *op;
+
+      op = op_data_new_authorize (uid, fstr);
+
+      g_queue_push_tail (ops, op);
+    }
+
+  allop_continue (client, task, ops);
+}
+
+gboolean
+bolt_client_authorize_all_finish (BoltClient   *client,
+                                  GAsyncResult *res,
+                                  GError      **error)
+{
+  g_autoptr(GError) err = NULL;
+  gboolean ok;
+
+  g_return_val_if_fail (BOLT_IS_CLIENT (client), FALSE);
+  g_return_val_if_fail (g_task_is_valid (res, client), FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  ok = g_task_propagate_boolean (G_TASK (res), &err);
+
+  if (!ok)
+    bolt_error_propagate_stripped (error, &err);
+
+  return ok;
+}
+
+void
+bolt_client_connect_all_async (BoltClient         *client,
+                               GPtrArray          *devices,
+                               BoltPolicy          policy,
+                               BoltAuthCtrl        flags,
+                               GCancellable       *cancellable,
+                               GAsyncReadyCallback callback,
+                               gpointer            user_data)
+{
+  g_autofree char *fstr = NULL;
+  GError *err = NULL;
+  const char *pstr;
+  GQueue *ops;
+  GTask *task;
+
+  g_return_if_fail (BOLT_IS_CLIENT (client));
+  g_return_if_fail (devices != NULL && devices->len > 0);
+  g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
+  g_return_if_fail (callback != NULL);
+
+  pstr = bolt_enum_to_string (BOLT_TYPE_POLICY, policy, &err);
+  if (pstr == NULL)
+    {
+      g_task_report_error (client, callback, user_data, NULL, err);
+      return;
+    }
+
+  fstr = bolt_flags_to_string (BOLT_TYPE_AUTH_CTRL, flags, &err);
+  if (fstr == NULL)
+    {
+      g_task_report_error (client, callback, user_data, NULL, err);
+      return;
+    }
+
+  task = g_task_new (client, cancellable, callback, user_data);
+  g_task_set_return_on_cancel (task, TRUE);
+
+  ops = g_queue_new ();
+  g_task_set_task_data (task, ops, (GDestroyNotify) op_queue_free);
+
+  for (guint i = 0; i < devices->len; i++)
+    {
+      BoltDevice *dev = g_ptr_array_index (devices, i);
+      const char *uid = bolt_device_get_uid (dev);
+      OpData *op;
+
+      if (bolt_device_is_stored (dev))
+        op = op_data_new_authorize (uid, fstr);
+      else
+        op = op_data_new_enroll (uid, pstr, fstr);
+
+      g_queue_push_tail (ops, op);
+    }
+
+  allop_continue (client, task, ops);
+}
+
+gboolean
+bolt_client_connect_all_finish (BoltClient   *client,
+                                GAsyncResult *res,
+                                GError      **error)
+{
+  g_autoptr(GError) err = NULL;
+  gboolean ok;
+
+  g_return_val_if_fail (BOLT_IS_CLIENT (client), FALSE);
+  g_return_val_if_fail (g_task_is_valid (res, client), FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  ok = g_task_propagate_boolean (G_TASK (res), &err);
+
+  if (!ok)
+    bolt_error_propagate_stripped (error, &err);
+
+  return ok;
+}
+
 gboolean
 bolt_client_forget_device (BoltClient *client,
                            const char *uid,
@@ -713,17 +1144,11 @@ bolt_client_new_power_client (BoltClient   *client,
 guint
 bolt_client_get_version (BoltClient *client)
 {
-  const char *key;
   guint val = 0;
-  gboolean ok;
 
   g_return_val_if_fail (BOLT_IS_CLIENT (client), val);
 
-  key = g_param_spec_get_name (props[PROP_VERSION]);
-  ok = bolt_proxy_get_property_uint32 (BOLT_PROXY (client), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get property '%s'", key);
+  val = bolt_proxy_get_uint32_by_pspec (client, props[PROP_VERSION]);
 
   return val;
 }
@@ -731,17 +1156,11 @@ bolt_client_get_version (BoltClient *client)
 gboolean
 bolt_client_is_probing (BoltClient *client)
 {
-  const char *key;
   gboolean val = FALSE;
-  gboolean ok;
 
   g_return_val_if_fail (BOLT_IS_CLIENT (client), val);
 
-  key = g_param_spec_get_name (props[PROP_PROBING]);
-  ok = bolt_proxy_get_property_bool (BOLT_PROXY (client), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_bool_by_pspec (client, props[PROP_PROBING]);
 
   return val;
 }
@@ -749,17 +1168,11 @@ bolt_client_is_probing (BoltClient *client)
 BoltPolicy
 bolt_client_get_policy (BoltClient *client)
 {
-  const char *key;
-  gboolean ok;
   gint val = BOLT_POLICY_UNKNOWN;
 
   g_return_val_if_fail (BOLT_IS_CLIENT (client), val);
 
-  key = g_param_spec_get_name (props[PROP_POLICY]);
-  ok = bolt_proxy_get_property_enum (BOLT_PROXY (client), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_enum_by_pspec (client, props[PROP_POLICY]);
 
   return val;
 }
@@ -767,17 +1180,11 @@ bolt_client_get_policy (BoltClient *client)
 BoltSecurity
 bolt_client_get_security (BoltClient *client)
 {
-  const char *key;
-  gboolean ok;
   gint val = BOLT_SECURITY_UNKNOWN;
 
   g_return_val_if_fail (BOLT_IS_CLIENT (client), val);
 
-  key = g_param_spec_get_name (props[PROP_SECURITY]);
-  ok = bolt_proxy_get_property_enum (BOLT_PROXY (client), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_enum_by_pspec (client, props[PROP_SECURITY]);
 
   return val;
 }
@@ -785,17 +1192,11 @@ bolt_client_get_security (BoltClient *client)
 BoltAuthMode
 bolt_client_get_authmode (BoltClient *client)
 {
-  const char *key;
-  gboolean ok;
   guint val = BOLT_AUTH_DISABLED;
 
   g_return_val_if_fail (BOLT_IS_CLIENT (client), val);
 
-  key = g_param_spec_get_name (props[PROP_AUTHMODE]);
-  ok = bolt_proxy_get_property_flags (BOLT_PROXY (client), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_flags_by_pspec (client, props[PROP_AUTHMODE]);
 
   return val;
 }
@@ -803,17 +1204,11 @@ bolt_client_get_authmode (BoltClient *client)
 BoltPowerState
 bolt_client_get_power_state (BoltClient *client)
 {
-  const char *key;
-  gboolean ok;
   gint val = BOLT_FORCE_POWER_UNSET;
 
   g_return_val_if_fail (BOLT_IS_CLIENT (client), val);
 
-  key = g_param_spec_get_name (props[PROP_POWERSTATE]);
-  ok = bolt_proxy_get_property_enum (BOLT_PROXY (client), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_enum_by_pspec (client, props[PROP_POWERSTATE]);
 
   return val;
 }
@@ -825,33 +1220,21 @@ bolt_client_set_authmode_async (BoltClient         *client,
                                 GAsyncReadyCallback callback,
                                 gpointer            user_data)
 {
-  g_autofree char *str = NULL;
-  GError *err = NULL;
-  GParamSpec *pspec;
-  GParamSpecFlags *flags_pspec;
-  GFlagsClass *flags_class;
+  g_auto(GValue) val = G_VALUE_INIT;
 
   g_return_if_fail (BOLT_IS_CLIENT (client));
   g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
   g_return_if_fail (callback != NULL);
 
-  pspec = props[PROP_AUTHMODE];
-  flags_pspec = G_PARAM_SPEC_FLAGS (pspec);
-  flags_class = flags_pspec->flags_class;
-  str = bolt_flags_class_to_string (flags_class, mode, &err);
+  g_value_init (&val, BOLT_TYPE_AUTH_MODE);
+  g_value_set_flags (&val, mode);
 
-  if (str == NULL)
-    {
-      g_task_report_error (client, callback, user_data, NULL, err);
-      return;
-    }
-
-  bolt_proxy_set_property_async (BOLT_PROXY (client),
-                                 g_param_spec_get_nick (pspec),
-                                 g_variant_new ("s", str),
-                                 cancellable,
-                                 callback,
-                                 user_data);
+  bolt_proxy_set_async (BOLT_PROXY (client),
+                        props[PROP_AUTHMODE],
+                        &val,
+                        cancellable,
+                        callback,
+                        user_data);
 }
 
 gboolean
