@@ -64,141 +64,131 @@ G_DEFINE_TYPE (BoltDevice,
                bolt_device,
                BOLT_TYPE_PROXY);
 
-static void
-bolt_device_get_property (GObject    *object,
-                          guint       prop_id,
-                          GValue     *value,
-                          GParamSpec *pspec)
-{
-  if (bolt_proxy_get_dbus_property (object, pspec, value))
-    return;
-}
-
-
 
 static void
 bolt_device_class_init (BoltDeviceClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  gobject_class->get_property = bolt_device_get_property;
+  gobject_class->get_property = bolt_proxy_property_getter;
+  gobject_class->set_property = bolt_proxy_property_setter;
 
   props[PROP_UID] =
-    g_param_spec_string ("uid",
-                         "Uid", NULL,
+    g_param_spec_string ("uid", "Uid",
+                         "The unique identifer.",
                          "unknown",
                          G_PARAM_READABLE |
-                         G_PARAM_STATIC_NICK);
+                         G_PARAM_STATIC_STRINGS);
 
   props[PROP_NAME] =
-    g_param_spec_string ("name",
-                         "Name", NULL,
+    g_param_spec_string ("name", "Name",
+                         "Human readable device name.",
                          "unknown",
                          G_PARAM_READABLE |
-                         G_PARAM_STATIC_NICK);
+                         G_PARAM_STATIC_STRINGS);
 
   props[PROP_VENDOR] =
-    g_param_spec_string ("vendor",
-                         "Vendor", NULL,
+    g_param_spec_string ("vendor", "Vendor",
+                         "The name of the device vendor",
                          "unknown",
                          G_PARAM_READABLE |
-                         G_PARAM_STATIC_NICK);
+                         G_PARAM_STATIC_STRINGS);
 
   props[PROP_TYPE] =
-    g_param_spec_enum ("type",
-                       "Type", NULL,
+    g_param_spec_enum ("type", "Type",
+                       "The type, i.e. host or peripheral.",
                        BOLT_TYPE_DEVICE_TYPE,
                        BOLT_DEVICE_PERIPHERAL,
                        G_PARAM_READABLE |
-                       G_PARAM_STATIC_NICK);
+                       G_PARAM_STATIC_STRINGS);
 
   props[PROP_STATUS] =
-    g_param_spec_enum ("status",
-                       "Status", NULL,
+    g_param_spec_enum ("status", "Status",
+                       "The device status.",
                        BOLT_TYPE_STATUS,
                        BOLT_STATUS_DISCONNECTED,
                        G_PARAM_READABLE |
-                       G_PARAM_STATIC_NICK);
+                       G_PARAM_STATIC_STRINGS);
 
   props[PROP_AUTHFLAGS] =
-    g_param_spec_flags ("authflags",
-                        "AuthFlags", NULL,
+    g_param_spec_flags ("authflags", "AuthFlags",
+                        "Flags describing the authentication state.",
                         BOLT_TYPE_AUTH_FLAGS,
                         BOLT_AUTH_NONE,
                         G_PARAM_READABLE |
                         G_PARAM_STATIC_STRINGS);
 
   props[PROP_PARENT] =
-    g_param_spec_string ("parent",
-                         "Parent", NULL,
+    g_param_spec_string ("parent", "Parent",
+                         "Unique identifier of the parent.",
                          "unknown",
                          G_PARAM_READABLE |
-                         G_PARAM_STATIC_NICK);
+                         G_PARAM_STATIC_STRINGS);
 
   props[PROP_SYSPATH] =
-    g_param_spec_string ("syspath",
-                         "SysfsPath", NULL,
+    g_param_spec_string ("syspath", "SysfsPath",
+                         "The sysfs path of the udev device.",
                          "unknown",
                          G_PARAM_READABLE |
-                         G_PARAM_STATIC_NICK);
+                         G_PARAM_STATIC_STRINGS);
 
   props[PROP_DOMAIN] =
-    g_param_spec_string ("domain",
-                         "Domain", NULL,
+    g_param_spec_string ("domain", "Domain",
+                         "Unique id of the corresponding domain.",
                          "unknown",
                          G_PARAM_READABLE |
-                         G_PARAM_STATIC_NICK);
+                         G_PARAM_STATIC_STRINGS);
 
 
   props[PROP_CONNTIME] =
-    g_param_spec_uint64 ("conntime",
-                         "ConnectTime", NULL,
+    g_param_spec_uint64 ("conntime", "ConnectTime",
+                         "When was the device connected?",
                          0, G_MAXUINT64, 0,
                          G_PARAM_READABLE |
                          G_PARAM_STATIC_STRINGS);
 
   props[PROP_AUTHTIME] =
-    g_param_spec_uint64 ("authtime",
-                         "AuthorizeTime", NULL,
+    g_param_spec_uint64 ("authtime", "AuthorizeTime",
+                         "When was the device authorized?",
                          0, G_MAXUINT64, 0,
                          G_PARAM_READABLE |
                          G_PARAM_STATIC_STRINGS);
 
   props[PROP_STORED] =
-    g_param_spec_boolean ("stored",
-                          "Stored", NULL,
+    g_param_spec_boolean ("stored", "Stored",
+                          "Is the device recorded in the database?",
                           FALSE,
                           G_PARAM_READABLE |
-                          G_PARAM_STATIC_NICK);
+                          G_PARAM_STATIC_STRINGS);
 
   props[PROP_POLICY] =
-    g_param_spec_enum ("policy",
-                       "Policy", NULL,
+    g_param_spec_enum ("policy", "Policy",
+                       "What to do when the device is connected?",
                        BOLT_TYPE_POLICY,
                        BOLT_POLICY_DEFAULT,
                        G_PARAM_READABLE |
-                       G_PARAM_STATIC_NICK);
+                       G_PARAM_STATIC_STRINGS);
 
   props[PROP_KEY] =
-    g_param_spec_enum ("key",
-                       "Key", NULL,
+    g_param_spec_enum ("key", "Key",
+                       "State of the device key.",
                        BOLT_TYPE_KEY_STATE,
                        BOLT_KEY_MISSING,
                        G_PARAM_READABLE |
-                       G_PARAM_STATIC_NICK);
+                       G_PARAM_STATIC_STRINGS);
 
   props[PROP_STORETIME] =
-    g_param_spec_uint64 ("storetime",
-                         "StoreTime", NULL,
+    g_param_spec_uint64 ("storetime", "StoreTime",
+                         "When was the device stored?",
                          0, G_MAXUINT64, 0,
                          G_PARAM_READABLE |
                          G_PARAM_STATIC_STRINGS);
 
   props[PROP_LABEL] =
-    g_param_spec_string ("label",
-                         "Label", NULL,
+    g_param_spec_string ("label", "Label",
+                         "The name given by bolt or the user.",
                          NULL,
-                         G_PARAM_READABLE |
+                         G_PARAM_READWRITE |
                          G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (gobject_class,
@@ -326,13 +316,11 @@ bolt_device_authorize_finish (BoltDevice   *dev,
 const char *
 bolt_device_get_uid (BoltDevice *dev)
 {
-  const char *key;
   const char *str;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), NULL);
 
-  key = g_param_spec_get_name (props[PROP_UID]);
-  str = bolt_proxy_get_property_string (BOLT_PROXY (dev), key);
+  str = bolt_proxy_get_string_by_pspec (dev, props[PROP_UID]);
 
   return str;
 }
@@ -340,13 +328,11 @@ bolt_device_get_uid (BoltDevice *dev)
 const char *
 bolt_device_get_name (BoltDevice *dev)
 {
-  const char *key;
   const char *str;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), NULL);
 
-  key = g_param_spec_get_name (props[PROP_NAME]);
-  str = bolt_proxy_get_property_string (BOLT_PROXY (dev), key);
+  str = bolt_proxy_get_string_by_pspec (dev, props[PROP_NAME]);
 
   return str;
 }
@@ -354,13 +340,11 @@ bolt_device_get_name (BoltDevice *dev)
 const char *
 bolt_device_get_vendor (BoltDevice *dev)
 {
-  const char *key;
   const char *str;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), NULL);
 
-  key = g_param_spec_get_name (props[PROP_VENDOR]);
-  str = bolt_proxy_get_property_string (BOLT_PROXY (dev), key);
+  str = bolt_proxy_get_string_by_pspec (dev, props[PROP_VENDOR]);
 
   return str;
 }
@@ -368,35 +352,31 @@ bolt_device_get_vendor (BoltDevice *dev)
 BoltDeviceType
 bolt_device_get_device_type (BoltDevice *dev)
 {
-  const char *key;
-  gboolean ok;
   gint val = BOLT_DEVICE_PERIPHERAL;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), val);
 
-  key = g_param_spec_get_name (props[PROP_TYPE]);
-  ok = bolt_proxy_get_property_enum (BOLT_PROXY (dev), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_enum_by_pspec (dev, props[PROP_TYPE]);
 
   return val;
 }
 
+gboolean
+bolt_device_is_host (BoltDevice *dev)
+{
+  BoltDeviceType dt = bolt_device_get_device_type (dev);
+
+  return dt == BOLT_DEVICE_HOST;
+}
+
 BoltStatus
 bolt_device_get_status (BoltDevice *dev)
 {
-  const char *key;
-  gboolean ok;
   gint val = BOLT_STATUS_UNKNOWN;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), val);
 
-  key = g_param_spec_get_name (props[PROP_STATUS]);
-  ok = bolt_proxy_get_property_enum (BOLT_PROXY (dev), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_enum_by_pspec (dev, props[PROP_STATUS]);
 
   return val;
 }
@@ -404,17 +384,11 @@ bolt_device_get_status (BoltDevice *dev)
 BoltAuthFlags
 bolt_device_get_authflags (BoltDevice *dev)
 {
-  const char *key;
-  gboolean ok;
   guint val = BOLT_AUTH_NONE;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), val);
 
-  key = g_param_spec_get_name (props[PROP_AUTHFLAGS]);
-  ok = bolt_proxy_get_property_flags (BOLT_PROXY (dev), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_flags_by_pspec (dev, props[PROP_AUTHFLAGS]);
 
   return val;
 }
@@ -422,13 +396,11 @@ bolt_device_get_authflags (BoltDevice *dev)
 const char *
 bolt_device_get_parent (BoltDevice *dev)
 {
-  const char *key;
   const char *str;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), NULL);
 
-  key = g_param_spec_get_name (props[PROP_PARENT]);
-  str = bolt_proxy_get_property_string (BOLT_PROXY (dev), key);
+  str = bolt_proxy_get_string_by_pspec (dev, props[PROP_PARENT]);
 
   return str;
 }
@@ -436,13 +408,11 @@ bolt_device_get_parent (BoltDevice *dev)
 const char *
 bolt_device_get_syspath (BoltDevice *dev)
 {
-  const char *key;
   const char *str;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), NULL);
 
-  key = g_param_spec_get_name (props[PROP_SYSPATH]);
-  str = bolt_proxy_get_property_string (BOLT_PROXY (dev), key);
+  str = bolt_proxy_get_string_by_pspec (dev, props[PROP_SYSPATH]);
 
   return str;
 }
@@ -450,30 +420,22 @@ bolt_device_get_syspath (BoltDevice *dev)
 const char *
 bolt_device_get_domain (BoltDevice *dev)
 {
-  const char *key;
   const char *str;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), NULL);
 
-  key = g_param_spec_get_name (props[PROP_DOMAIN]);
-  str = bolt_proxy_get_property_string (BOLT_PROXY (dev), key);
+  str = bolt_proxy_get_string_by_pspec (dev, props[PROP_DOMAIN]);
 
   return str;
 }
 guint64
 bolt_device_get_conntime (BoltDevice *dev)
 {
-  const char *key;
   guint64 val = 0;
-  gboolean ok;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), val);
 
-  key = g_param_spec_get_name (props[PROP_CONNTIME]);
-  ok = bolt_proxy_get_property_uint64 (BOLT_PROXY (dev), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_uint64_by_pspec (dev, props[PROP_CONNTIME]);
 
   return val;
 }
@@ -481,17 +443,11 @@ bolt_device_get_conntime (BoltDevice *dev)
 guint64
 bolt_device_get_authtime (BoltDevice *dev)
 {
-  const char *key;
   guint64 val = 0;
-  gboolean ok;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), val);
 
-  key = g_param_spec_get_name (props[PROP_AUTHTIME]);
-  ok = bolt_proxy_get_property_uint64 (BOLT_PROXY (dev), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_uint64_by_pspec (dev, props[PROP_AUTHTIME]);
 
   return val;
 }
@@ -499,17 +455,11 @@ bolt_device_get_authtime (BoltDevice *dev)
 gboolean
 bolt_device_is_stored (BoltDevice *dev)
 {
-  const char *key;
   gboolean val = FALSE;
-  gboolean ok;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), val);
 
-  key = g_param_spec_get_name (props[PROP_STORED]);
-  ok = bolt_proxy_get_property_bool (BOLT_PROXY (dev), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_bool_by_pspec (dev, props[PROP_STORED]);
 
   return val;
 }
@@ -517,17 +467,11 @@ bolt_device_is_stored (BoltDevice *dev)
 BoltPolicy
 bolt_device_get_policy (BoltDevice *dev)
 {
-  const char *key;
-  gboolean ok;
   gint val = BOLT_POLICY_DEFAULT;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), val);
 
-  key = g_param_spec_get_name (props[PROP_POLICY]);
-  ok = bolt_proxy_get_property_enum (BOLT_PROXY (dev), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_enum_by_pspec (dev, props[PROP_POLICY]);
 
   return val;
 }
@@ -535,17 +479,11 @@ bolt_device_get_policy (BoltDevice *dev)
 BoltKeyState
 bolt_device_get_keystate (BoltDevice *dev)
 {
-  const char *key;
-  gboolean ok;
   gint val = BOLT_KEY_MISSING;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), val);
 
-  key = g_param_spec_get_name (props[PROP_KEY]);
-  ok = bolt_proxy_get_property_enum (BOLT_PROXY (dev), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_enum_by_pspec (dev, props[PROP_KEY]);
 
   return val;
 }
@@ -553,17 +491,11 @@ bolt_device_get_keystate (BoltDevice *dev)
 guint64
 bolt_device_get_storetime (BoltDevice *dev)
 {
-  const char *key;
   guint64 val = 0;
-  gboolean ok;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), val);
 
-  key = g_param_spec_get_name (props[PROP_STORETIME]);
-  ok = bolt_proxy_get_property_uint64 (BOLT_PROXY (dev), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_uint64_by_pspec (dev, props[PROP_STORETIME]);
 
   return val;
 }
@@ -571,13 +503,11 @@ bolt_device_get_storetime (BoltDevice *dev)
 const char *
 bolt_device_get_label (BoltDevice *dev)
 {
-  const char *key;
   const char *str;
 
   g_return_val_if_fail (BOLT_IS_DEVICE (dev), NULL);
 
-  key = g_param_spec_get_name (props[PROP_LABEL]);
-  str = bolt_proxy_get_property_string (BOLT_PROXY (dev), key);
+  str = bolt_proxy_get_string_by_pspec (dev, props[PROP_LABEL]);
 
   return str;
 }
