@@ -38,6 +38,7 @@ enum {
   PROP_SYSPATH,
   PROP_SECURITY,
   PROP_BOOTACL,
+  PROP_IOMMU,
 
   PROP_LAST
 };
@@ -48,60 +49,57 @@ G_DEFINE_TYPE (BoltDomain,
                bolt_domain,
                BOLT_TYPE_PROXY);
 
-static void
-bolt_domain_get_property (GObject    *object,
-                          guint       prop_id,
-                          GValue     *value,
-                          GParamSpec *pspec)
-{
-  if (bolt_proxy_get_dbus_property (object, pspec, value))
-    return;
-}
-
-
 
 static void
 bolt_domain_class_init (BoltDomainClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  gobject_class->get_property = bolt_domain_get_property;
+  gobject_class->get_property = bolt_proxy_property_getter;
+  gobject_class->set_property = bolt_proxy_property_setter;
 
   props[PROP_UID] =
-    g_param_spec_string ("uid",
-                         "Uid", NULL,
+    g_param_spec_string ("uid", "Uid",
+                         "The unique identifier.",
                          "unknown",
                          G_PARAM_READABLE |
-                         G_PARAM_STATIC_NICK);
+                         G_PARAM_STATIC_STRINGS);
 
   props[PROP_ID] =
-    g_param_spec_string ("id",
-                         "Id", NULL,
+    g_param_spec_string ("id", "Id",
+                         "The sysfs name.",
                          "unknown",
                          G_PARAM_READABLE |
-                         G_PARAM_STATIC_NICK);
+                         G_PARAM_STATIC_STRINGS);
 
   props[PROP_SYSPATH] =
-    g_param_spec_string ("syspath",
-                         "SysfsPath", NULL,
+    g_param_spec_string ("syspath", "SysfsPath",
+                         "Sysfs path of the udev device.",
                          "unknown",
                          G_PARAM_READABLE |
-                         G_PARAM_STATIC_NICK);
+                         G_PARAM_STATIC_STRINGS);
 
   props[PROP_SECURITY]
-    = g_param_spec_enum ("security",
-                         "SecurityLevel", NULL,
+    = g_param_spec_enum ("security", "SecurityLevel",
+                         "The security level set in the BIOS.",
                          BOLT_TYPE_SECURITY,
                          BOLT_SECURITY_UNKNOWN,
                          G_PARAM_READABLE |
-                         G_PARAM_STATIC_NAME);
+                         G_PARAM_STATIC_STRINGS);
 
   props[PROP_BOOTACL] =
-    g_param_spec_boxed ("bootacl",
-                        "BootACL", NULL,
+    g_param_spec_boxed ("bootacl", "BootACL",
+                        "Pre-boot access control list (uuids).",
                         G_TYPE_STRV,
                         G_PARAM_READABLE |
-                        G_PARAM_STATIC_NAME);
+                        G_PARAM_STATIC_STRINGS);
+
+  props[PROP_IOMMU] =
+    g_param_spec_boolean ("iommu", "IOMMU",
+                          "Is IOMMU based DMA protection active?",
+                          FALSE,
+                          G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (gobject_class,
                                      PROP_LAST,
@@ -143,13 +141,11 @@ bolt_domain_new_for_object_path (GDBusConnection *bus,
 const char *
 bolt_domain_get_uid (BoltDomain *domain)
 {
-  const char *key;
   const char *str;
 
   g_return_val_if_fail (BOLT_IS_DOMAIN (domain), NULL);
 
-  key = g_param_spec_get_name (props[PROP_UID]);
-  str = bolt_proxy_get_property_string (BOLT_PROXY (domain), key);
+  str = bolt_proxy_get_string_by_pspec (domain, props[PROP_UID]);
 
   return str;
 }
@@ -157,13 +153,11 @@ bolt_domain_get_uid (BoltDomain *domain)
 const char *
 bolt_domain_get_id (BoltDomain *domain)
 {
-  const char *key;
   const char *str;
 
   g_return_val_if_fail (BOLT_IS_DOMAIN (domain), NULL);
 
-  key = g_param_spec_get_name (props[PROP_ID]);
-  str = bolt_proxy_get_property_string (BOLT_PROXY (domain), key);
+  str = bolt_proxy_get_string_by_pspec (domain, props[PROP_ID]);
 
   return str;
 }
@@ -171,13 +165,11 @@ bolt_domain_get_id (BoltDomain *domain)
 const char *
 bolt_domain_get_syspath (BoltDomain *domain)
 {
-  const char *key;
   const char *str;
 
   g_return_val_if_fail (BOLT_IS_DOMAIN (domain), NULL);
 
-  key = g_param_spec_get_name (props[PROP_SYSPATH]);
-  str = bolt_proxy_get_property_string (BOLT_PROXY (domain), key);
+  str = bolt_proxy_get_string_by_pspec (domain, props[PROP_SYSPATH]);
 
   return str;
 }
@@ -185,17 +177,11 @@ bolt_domain_get_syspath (BoltDomain *domain)
 BoltSecurity
 bolt_domain_get_security (BoltDomain *domain)
 {
-  const char *key;
-  gboolean ok;
   gint val = BOLT_SECURITY_UNKNOWN;
 
   g_return_val_if_fail (BOLT_IS_DOMAIN (domain), val);
 
-  key = g_param_spec_get_name (props[PROP_SECURITY]);
-  ok = bolt_proxy_get_property_enum (BOLT_PROXY (domain), key, &val);
-
-  if (!ok)
-    g_warning ("failed to get enum property '%s'", key);
+  val = bolt_proxy_get_enum_by_pspec (domain, props[PROP_SECURITY]);
 
   return val;
 }
@@ -203,13 +189,11 @@ bolt_domain_get_security (BoltDomain *domain)
 char **
 bolt_domain_get_bootacl (BoltDomain *domain)
 {
-  const char *key;
   char **strv;
 
   g_return_val_if_fail (BOLT_IS_DOMAIN (domain), NULL);
 
-  key = g_param_spec_get_name (props[PROP_BOOTACL]);
-  strv = bolt_proxy_get_property_strv (BOLT_PROXY (domain), key);
+  strv = bolt_proxy_get_strv_by_pspec (domain, props[PROP_BOOTACL]);
 
   return strv;
 }
@@ -223,3 +207,15 @@ bolt_domain_is_online (BoltDomain *domain)
 
   return syspath != NULL;
 }
+
+gboolean
+bolt_domain_has_iommu (BoltDomain *domain)
+{
+  gboolean val = FALSE;
+
+  g_return_val_if_fail (BOLT_IS_DOMAIN (domain), val);
+
+  val = bolt_proxy_get_bool_by_pspec (domain, props[PROP_IOMMU]);
+
+  return val;
+}
