@@ -20,8 +20,10 @@
 
 #include "bolt-proxy.h"
 
+#include "bolt-dbus.h"
 #include "bolt-enums.h"
 #include "bolt-error.h"
+#include "bolt-glue.h"
 #include "bolt-names.h"
 #include "bolt-str.h"
 
@@ -47,12 +49,57 @@ enum {
   PROP_LAST
 };
 
-G_DEFINE_TYPE (BoltProxy, bolt_proxy, G_TYPE_DBUS_PROXY);
+struct _BoltProxyClassPrivate
+{
+  GHashTable *wire_convs;
+};
+
+static gpointer bolt_proxy_parent_class = NULL;
+
+static void     bolt_proxy_init (GTypeInstance *,
+                                 gpointer g_class);
+static void     bolt_proxy_class_init (BoltProxyClass *klass);
+static void     bolt_proxy_base_init (gpointer g_class);
+static void     bolt_proxy_base_finalize (gpointer g_class);
+
+GType
+bolt_proxy_get_type (void)
+{
+  static volatile gsize proxy_type = 0;
+
+  if (g_once_init_enter (&proxy_type))
+    {
+      GType type_id;
+      const GTypeInfo type_info = {
+        sizeof (BoltProxyClass),
+        bolt_proxy_base_init,
+        (GBaseFinalizeFunc) bolt_proxy_base_finalize,
+        (GClassInitFunc) bolt_proxy_class_init,
+        NULL,           /* class_finalize */
+        NULL,           /* class_data */
+        sizeof (BoltProxy),
+        0,              /* n_preallocs */
+        bolt_proxy_init,
+        NULL,           /* value_table */
+      };
+
+      type_id = g_type_register_static (G_TYPE_DBUS_PROXY, "BoltProxy",
+                                        &type_info, G_TYPE_FLAG_ABSTRACT);
+      g_type_add_class_private (type_id, sizeof (BoltProxyClassPrivate));
+      g_once_init_leave (&proxy_type, type_id);
+    }
+
+  return proxy_type;
+}
 
 
 static void
 bolt_proxy_constructed (GObject *object)
 {
+  g_autoptr(GError) err = NULL;
+  GDBusInterfaceInfo *info;
+  const char *interface;
+
   G_OBJECT_CLASS (bolt_proxy_parent_class)->constructed (object);
 
   g_signal_connect (object, "g-properties-changed",
@@ -60,6 +107,29 @@ bolt_proxy_constructed (GObject *object)
 
   g_signal_connect (object, "g-signal",
                     G_CALLBACK (bolt_proxy_handle_dbus_signal), object);
+
+  if (g_dbus_proxy_get_interface_info (G_DBUS_PROXY (object)) != NULL)
+    return;
+
+  interface = g_dbus_proxy_get_interface_name (G_DBUS_PROXY (object));
+
+  if (interface == NULL)
+    return;
+
+  bolt_dbus_ensure_resources ();
+
+  info = bolt_dbus_interface_info_lookup (BOLT_DBUS_GRESOURCE_PATH,
+                                          interface,
+                                          &err);
+
+  if (info == NULL)
+    {
+      g_warning ("could not load interface info: %s", err->message);
+      return;
+    }
+
+  g_dbus_proxy_set_interface_info (G_DBUS_PROXY (object), info);
+  g_dbus_interface_info_unref (info);
 }
 
 static const BoltProxySignal *
@@ -74,6 +144,8 @@ bolt_proxy_class_init (BoltProxyClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
+  bolt_proxy_parent_class = g_type_class_peek_parent (klass);
+
   gobject_class->constructed = bolt_proxy_constructed;
 
   klass->get_dbus_signals = bolt_proxy_get_dbus_signals;
@@ -81,7 +153,29 @@ bolt_proxy_class_init (BoltProxyClass *klass)
 }
 
 static void
-bolt_proxy_init (BoltProxy *object)
+bolt_proxy_base_init (gpointer g_class)
+{
+  BoltProxyClass *klass = g_class;
+
+  klass->priv = G_TYPE_CLASS_GET_PRIVATE (g_class, BOLT_TYPE_PROXY, BoltProxyClassPrivate);
+  memset (klass->priv, 0, sizeof (BoltProxyClassPrivate));
+
+  klass->priv->wire_convs = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                   NULL,
+                                                   (GDestroyNotify) bolt_wire_conv_unref);
+}
+
+static void
+bolt_proxy_base_finalize (gpointer g_class)
+{
+  BoltProxyClass *klass = g_class;
+  BoltProxyClassPrivate *priv = klass->priv;
+
+  g_hash_table_unref (priv->wire_convs);
+}
+
+static void
+bolt_proxy_init (GTypeInstance *instance, gpointer g_class)
 {
 }
 
@@ -150,65 +244,114 @@ bolt_proxy_handle_dbus_signal (GDBusProxy  *proxy,
 
 }
 
+static BoltWireConv *
+bolt_proxy_get_wire_conv (BoltProxy  *proxy,
+                          GParamSpec *spec,
+                          GError    **error)
+{
+  GDBusInterfaceInfo *info;
+  GDBusPropertyInfo *pi;
+  BoltProxyClass *klass;
+  const char *nick;
+  BoltWireConv *conv;
+
+  klass = BOLT_PROXY_GET_CLASS (proxy);
+
+  nick = g_param_spec_get_nick (spec);
+  conv = g_hash_table_lookup (klass->priv->wire_convs, nick);
+
+  if (conv)
+    return conv;
+
+  info = g_dbus_proxy_get_interface_info (G_DBUS_PROXY (proxy));
+
+  if (info == NULL)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "could not find dbus interface info");
+      return NULL;
+    }
+
+  pi = g_dbus_interface_info_lookup_property (info, nick);
+  if (pi == NULL)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
+                   "could not find dbus property info");
+      return NULL;
+    }
+
+  conv = bolt_wire_conv_for (G_VARIANT_TYPE (pi->signature), spec);
+
+  if (conv == NULL)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "could create conversion helper");
+      return NULL;
+    }
+
+  /* nick is valid as long as spec is valid and
+   * a reference to spec is held by conv */
+  g_hash_table_insert (klass->priv->wire_convs,
+                       (gpointer) nick, conv);
+
+  return conv;
+}
+
 /* public methods */
+void
+bolt_proxy_property_getter (GObject    *object,
+                            guint       prop_id,
+                            GValue     *value,
+                            GParamSpec *spec)
+{
+  bolt_proxy_get_dbus_property (BOLT_PROXY (object),
+                                spec,
+                                value);
+}
 
 gboolean
-bolt_proxy_get_dbus_property (GObject    *proxy,
+bolt_proxy_get_dbus_property (BoltProxy  *proxy,
                               GParamSpec *spec,
                               GValue     *value)
 {
   g_autoptr(GVariant) val = NULL;
-  const GVariantType *vt;
-  gboolean handled = FALSE;
+  g_autoptr(GError) err = NULL;
+  BoltWireConv *conv;
   const char *nick;
+  gboolean ok;
 
   nick = g_param_spec_get_nick (spec);
   val = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), nick);
 
   if (val == NULL)
-    return FALSE;
-
-  vt = g_variant_get_type (val);
-
-  if (g_variant_type_equal (vt, G_VARIANT_TYPE_STRING) &&
-      G_IS_PARAM_SPEC_ENUM (spec))
     {
-      GParamSpecEnum *enum_spec = G_PARAM_SPEC_ENUM (spec);
-      GEnumValue *ev;
-      const char *str;
+      const GValue *def = g_param_spec_get_default_value (spec);
 
-      str = g_variant_get_string (val, NULL);
-      ev = g_enum_get_value_by_nick (enum_spec->enum_class, str);
+      if (G_VALUE_TYPE (value) == 0)
+        g_value_init (value, spec->value_type);
+      g_value_copy (def, value);
 
-      handled = ev != NULL;
+      g_warning ("Unknown property: %s (%s)", spec->name, nick);
 
-      if (handled)
-        g_value_set_enum (value, ev->value);
-      else
-        g_value_set_enum (value, enum_spec->default_value);
-    }
-  else if (g_variant_type_equal (vt, G_VARIANT_TYPE_STRING) &&
-           G_IS_PARAM_SPEC_FLAGS (spec))
-    {
-      GParamSpecFlags *flags_spec = G_PARAM_SPEC_FLAGS (spec);
-      GFlagsClass *flags_class = flags_spec->flags_class;
-      const char *str;
-      guint v;
-
-      str = g_variant_get_string (val, NULL);
-      handled = bolt_flags_class_from_string (flags_class, str, &v, NULL);
-
-      if (handled)
-        g_value_set_flags (value, v);
-      else
-        g_value_set_flags (value, flags_spec->default_value);
-    }
-  else
-    {
-      g_dbus_gvariant_to_gvalue (val, value);
+      return FALSE;
     }
 
-  return handled;
+  conv = bolt_proxy_get_wire_conv (proxy, spec, &err);
+
+  if (conv == NULL)
+    {
+      g_warning ("No conversion available for dbus property '%s': %s",
+                 nick, err->message);
+      return FALSE;
+    }
+
+  ok = bolt_wire_conv_from_wire (conv, val, value, &err);
+
+  if (!ok)
+    g_warning ("Failed to convert dbus property '%s': %s",
+               nick, err->message);
+
+  return ok;
 }
 
 const char *
@@ -260,205 +403,123 @@ find_property (BoltProxy  *proxy,
   return res;
 }
 
-static GVariant *
-bolt_proxy_get_cached_property (BoltProxy  *proxy,
-                                const char *name)
-{
-  const char *bus_name = NULL;
-  GParamSpec *pspec;
-  GVariant *var;
-
-  g_return_val_if_fail (BOLT_IS_PROXY (proxy), NULL);
-  g_return_val_if_fail (name != NULL, NULL);
-
-  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (proxy), name);
-
-  if (pspec == NULL)
-    return NULL;
-
-  bus_name = g_param_spec_get_nick (pspec);
-  var = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), bus_name);
-
-  return var;
-}
-
 gboolean
-bolt_proxy_get_property_bool (BoltProxy  *proxy,
-                              const char *name,
-                              gboolean   *value)
+bolt_proxy_get_bool_by_pspec (gpointer    object,
+                              GParamSpec *spec)
 {
-  g_autoptr(GVariant) var = NULL;
-
-  var = bolt_proxy_get_cached_property (proxy, name);
-
-  if (var == NULL)
-    return FALSE;
-  else if (value)
-    *value = g_variant_get_boolean (var);
-
-  return TRUE;
-}
-
-gboolean
-bolt_proxy_get_property_enum (BoltProxy  *proxy,
-                              const char *name,
-                              gint       *value)
-{
-  g_autoptr(GVariant) var = NULL;
-  const char *str = NULL;
-  const char *bus_name = NULL;
-  GParamSpec *pspec;
-  GEnumValue *ev;
+  g_auto(GValue) val = G_VALUE_INIT;
+  BoltProxy *proxy = object;
 
   g_return_val_if_fail (BOLT_IS_PROXY (proxy), FALSE);
 
-  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (proxy), name);
+  bolt_proxy_get_dbus_property (proxy, spec, &val);
 
-  if (pspec == NULL)
-    return FALSE;
-
-  bus_name = g_param_spec_get_nick (pspec);
-  var = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), bus_name);
-  if (var == NULL)
-    return FALSE;
-
-  str = g_variant_get_string (var, NULL);
-
-  if (str == NULL)
-    return FALSE;
-
-  ev = g_enum_get_value_by_nick (G_PARAM_SPEC_ENUM (pspec)->enum_class, str);
-
-  if (ev == NULL)
-    return FALSE;
-
-  if (value)
-    *value = ev->value;
-
-  return TRUE;
+  return g_value_get_boolean (&val);
 }
 
-gboolean
-bolt_proxy_get_property_flags (BoltProxy  *proxy,
-                               const char *name,
-                               guint      *value)
+
+gint
+bolt_proxy_get_enum_by_pspec (gpointer    object,
+                              GParamSpec *spec)
 {
-  g_autoptr(GVariant) var = NULL;
-  const char *str = NULL;
-  const char *bus_name = NULL;
-  GFlagsClass *flags_class;
-  GParamSpec *pspec;
-  guint v;
-  gboolean ok;
+  g_auto(GValue) val = G_VALUE_INIT;
+  BoltProxy *proxy = (BoltProxy *) object;
 
   g_return_val_if_fail (BOLT_IS_PROXY (proxy), FALSE);
 
-  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (proxy), name);
+  bolt_proxy_get_dbus_property (proxy, spec, &val);
 
-  if (pspec == NULL || !G_IS_PARAM_SPEC_FLAGS (pspec))
-    return FALSE;
-
-  bus_name = g_param_spec_get_nick (pspec);
-  var = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), bus_name);
-  if (var == NULL)
-    return FALSE;
-
-  str = g_variant_get_string (var, NULL);
-
-  if (str == NULL)
-    return FALSE;
-
-  flags_class = G_PARAM_SPEC_FLAGS (pspec)->flags_class;
-  ok = bolt_flags_class_from_string (flags_class, str, &v, NULL);
-
-  if (ok && value)
-    *value = v;
-
-  return ok;
+  return g_value_get_enum (&val);
 }
 
-gboolean
-bolt_proxy_get_property_uint32 (BoltProxy  *proxy,
-                                const char *name,
-                                guint      *value)
+guint
+bolt_proxy_get_flags_by_pspec (gpointer    object,
+                               GParamSpec *spec)
 {
-  g_autoptr(GVariant) var = NULL;
+  g_auto(GValue) val = G_VALUE_INIT;
+  BoltProxy *proxy = (BoltProxy *) object;
 
-  var = bolt_proxy_get_cached_property (proxy, name);
+  g_return_val_if_fail (BOLT_IS_PROXY (proxy), FALSE);
 
-  if (var == NULL)
-    return FALSE;
-  else if (value)
-    *value = g_variant_get_uint32 (var);
+  bolt_proxy_get_dbus_property (proxy, spec, &val);
 
-  return TRUE;
+  return g_value_get_flags (&val);
 }
 
-gboolean
-bolt_proxy_get_property_int64 (BoltProxy  *proxy,
-                               const char *name,
-                               gint64     *value)
+guint32
+bolt_proxy_get_uint32_by_pspec (gpointer    object,
+                                GParamSpec *spec)
 {
-  g_autoptr(GVariant) var = NULL;
+  g_auto(GValue) val = G_VALUE_INIT;
+  BoltProxy *proxy = (BoltProxy *) object;
 
-  var = bolt_proxy_get_cached_property (proxy, name);
+  g_return_val_if_fail (BOLT_IS_PROXY (proxy), FALSE);
 
-  if (var == NULL)
-    return FALSE;
-  else if (value)
-    *value = g_variant_get_int64 (var);
+  bolt_proxy_get_dbus_property (proxy, spec, &val);
 
-  return TRUE;
+  return g_value_get_uint (&val);
 }
 
-gboolean
-bolt_proxy_get_property_uint64 (BoltProxy  *proxy,
-                                const char *name,
-                                guint64    *value)
+gint64
+bolt_proxy_get_int64_by_pspec (gpointer    object,
+                               GParamSpec *spec)
 {
-  g_autoptr(GVariant) var = NULL;
+  g_auto(GValue) val = G_VALUE_INIT;
+  BoltProxy *proxy = (BoltProxy *) object;
 
-  var = bolt_proxy_get_cached_property (proxy, name);
+  g_return_val_if_fail (BOLT_IS_PROXY (proxy), FALSE);
 
-  if (var == NULL)
-    return FALSE;
-  else if (value)
-    *value = g_variant_get_uint64 (var);
+  bolt_proxy_get_dbus_property (proxy, spec, &val);
 
-  return TRUE;
+  return g_value_get_int64 (&val);
+}
+
+guint64
+bolt_proxy_get_uint64_by_pspec (gpointer    object,
+                                GParamSpec *spec)
+{
+  g_auto(GValue) val = G_VALUE_INIT;
+  BoltProxy *proxy = (BoltProxy *) object;
+
+  g_return_val_if_fail (BOLT_IS_PROXY (proxy), FALSE);
+
+  bolt_proxy_get_dbus_property (proxy, spec, &val);
+
+  return g_value_get_uint64 (&val);
 }
 
 const char *
-bolt_proxy_get_property_string (BoltProxy  *proxy,
-                                const char *name)
+bolt_proxy_get_string_by_pspec (gpointer    object,
+                                GParamSpec *spec)
 {
-  g_autoptr(GVariant) var = NULL;
-  const char *val = NULL;
+  g_auto(GValue) val = G_VALUE_INIT;
+  BoltProxy *proxy = (BoltProxy *) object;
 
-  var = bolt_proxy_get_cached_property (proxy, name);
+  g_return_val_if_fail (BOLT_IS_PROXY (proxy), FALSE);
 
-  if (var != NULL)
-    val = g_variant_get_string (var, NULL);
+  bolt_proxy_get_dbus_property (proxy, spec, &val);
 
-  if (val && *val == '\0')
-    val = NULL;
-
-  return val;
+  /* NB: Yes, this is return the string inside the value,
+   * although the value is freed. This therefore makes
+   * the assumption that the GValue's value was initialized
+   * via a static string (g_value_set_static_string). Ergo,
+   * this must always be true for bolt_proxy_get_dbus_property
+   */
+  return g_value_get_string (&val);
 }
 
 char **
-bolt_proxy_get_property_strv (BoltProxy  *proxy,
-                              const char *name)
+bolt_proxy_get_strv_by_pspec (gpointer    object,
+                              GParamSpec *spec)
 {
-  g_autoptr(GVariant) var = NULL;
-  char **val = NULL;
+  g_auto(GValue) val = G_VALUE_INIT;
+  BoltProxy *proxy = (BoltProxy *) object;
 
-  var = bolt_proxy_get_cached_property (proxy, name);
+  g_return_val_if_fail (BOLT_IS_PROXY (proxy), FALSE);
 
-  if (var != NULL)
-    val = g_variant_dup_strv (var, NULL);
+  bolt_proxy_get_dbus_property (proxy, spec, &val);
 
-  return val;
+  return (char **) g_value_dup_boxed (&val);
 }
 
 gboolean
@@ -546,3 +607,141 @@ bolt_proxy_set_property_finish (GAsyncResult *res,
   g_variant_unref (val);
   return TRUE;
 }
+
+gboolean
+bolt_proxy_set (BoltProxy    *proxy,
+                GParamSpec   *spec,
+                const GValue *value,
+                GCancellable *cancellable,
+                GError      **error)
+{
+  g_autoptr(GVariant) val = NULL;
+  g_autoptr(GVariant) res = NULL;
+  BoltWireConv *conv;
+  const char *name;
+  const char *iface;
+
+  g_return_val_if_fail (BOLT_IS_PROXY (proxy), FALSE);
+  g_return_val_if_fail (G_IS_PARAM_SPEC (spec), FALSE);
+  g_return_val_if_fail (G_IS_VALUE (value), FALSE);
+  g_return_val_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable), FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  conv = bolt_proxy_get_wire_conv (proxy, spec, error);
+
+  if (conv == NULL)
+    return FALSE;
+
+  val = bolt_wire_conv_to_wire (conv, value, error);
+
+  if (val == NULL)
+    return FALSE;
+
+  name = g_param_spec_get_nick (spec);
+  iface = g_dbus_proxy_get_interface_name (G_DBUS_PROXY (proxy));
+
+  res = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
+                                "org.freedesktop.DBus.Properties.Set",
+                                g_variant_new ("(ssv)",
+                                               iface,
+                                               name,
+                                               val),
+                                G_DBUS_CALL_FLAGS_NONE,
+                                -1,
+                                cancellable,
+                                error);
+
+  return res != NULL;
+}
+
+void
+bolt_proxy_set_async (BoltProxy          *proxy,
+                      GParamSpec         *spec,
+                      const GValue       *value,
+                      GCancellable       *cancellable,
+                      GAsyncReadyCallback callback,
+                      gpointer            user_data)
+{
+  g_autoptr(GVariant) val = NULL;
+  GError *err;
+  BoltWireConv *conv;
+  const char *name;
+  const char *iface;
+
+  g_return_if_fail (BOLT_IS_PROXY (proxy));
+  g_return_if_fail (G_IS_PARAM_SPEC (spec));
+  g_return_if_fail (G_IS_VALUE (value));
+  g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
+
+  conv = bolt_proxy_get_wire_conv (proxy, spec, &err);
+
+  if (conv != NULL)
+    val = bolt_wire_conv_to_wire (conv, value, &err);
+
+  if (val == NULL)
+    {
+      g_task_report_error (proxy, callback, user_data, NULL, err);
+      return;
+    }
+
+  name = g_param_spec_get_nick (spec);
+  iface = g_dbus_proxy_get_interface_name (G_DBUS_PROXY (proxy));
+
+  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
+                     "org.freedesktop.DBus.Properties.Set",
+                     g_variant_new ("(ssv)",
+                                    iface,
+                                    name,
+                                    val),
+                     G_DBUS_CALL_FLAGS_NONE,
+                     -1,
+                     cancellable,
+                     callback,
+                     user_data);
+}
+
+gboolean
+bolt_proxy_set_finish (GAsyncResult *res,
+                       GError      **error)
+{
+  g_autoptr(GVariant) val = NULL;
+  BoltProxy *proxy;
+
+  proxy = (BoltProxy *) g_async_result_get_source_object (res);
+  val = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
+
+  return val != NULL;
+}
+
+static void
+set_async_done (GObject      *source_object,
+                GAsyncResult *res,
+                gpointer      user_data)
+{
+  g_autoptr(GError) err = NULL;
+  g_autoptr(GParamSpec) spec = user_data;
+  gboolean ok;
+
+  ok = bolt_proxy_set_finish (res, &err);
+
+  if (!ok)
+    g_warning ("Error setting property '%s': %s",
+               spec->name, err->message);
+}
+
+void
+bolt_proxy_property_setter (GObject      *object,
+                            guint         prop_id,
+                            const GValue *value,
+                            GParamSpec   *spec)
+{
+  g_return_if_fail (BOLT_IS_PROXY (object));
+  g_return_if_fail (value != NULL);
+  g_return_if_fail (G_IS_PARAM_SPEC (spec));
+
+  bolt_proxy_set_async (BOLT_PROXY (object),
+                        spec, value,
+                        NULL,
+                        set_async_done,
+                        g_param_spec_ref (spec));
+}
