@@ -179,7 +179,10 @@ typedef enum {
 BoltSecurity     bolt_security_from_string (const char *str);
 const char *     bolt_security_to_string (BoltSecurity security);
 gboolean         bolt_security_allows_pcie (BoltSecurity security);
+gboolean         bolt_security_is_interactive (BoltSecurity security);
 
+const char *     bolt_security_for_display (BoltSecurity security,
+                                            gboolean     iommu);
 /**
  * BoltPolicy:
  * @BOLT_POLICY_UNKNOWN: Unknown policy.
@@ -188,6 +191,10 @@ gboolean         bolt_security_allows_pcie (BoltSecurity security);
  * @BOLT_POLICY_AUTO: Connect the device automatically,
  *   with the best possible security level supported
  *   by the domain controller.
+ * @BOLT_POLICY_IOMMU: Like @BOLT_POLICY_AUTO if IOMMU, i.e.
+ *   hardware DMA protection, is active and like  @BOLT_POLICY_MANUAL
+ *   otherwise: automatically authorize the device if and only if
+ *   we have IOMMU active support.
  *
  * What do to for connected devices.
  */
@@ -197,6 +204,7 @@ typedef enum {
   BOLT_POLICY_DEFAULT = 0,
   BOLT_POLICY_MANUAL = 1,
   BOLT_POLICY_AUTO = 2,
+  BOLT_POLICY_IOMMU = 3,
 
 } BoltPolicy;
 
@@ -254,6 +262,8 @@ typedef enum { /*< flags >*/
 #define bolt_auth_mode_is_enabled(auth) ((auth & BOLT_AUTH_ENABLED) != 0)
 #define bolt_auth_mode_is_disabled(auth) (!bolt_auth_mode_is_enabled (auth))
 
+char *  bolt_auth_mode_to_string (BoltAuthMode mode);
+
 /**
  * BoltPowerState:
  * @BOLT_FORCE_POWER_UNSET: Force power was not set by bolt.
