@@ -20,6 +20,10 @@
 
 #pragma once
 
+#include <glib.h>
+
+G_BEGIN_DECLS
+
 /* D-Bus API revision (here for the lack of a better place) */
 #define BOLT_DBUS_API_VERSION 1U
 
@@ -47,6 +51,7 @@
 
 
 /* dbus */
+#define BOLT_DBUS_GRESOURCE_PATH "/bolt/org.freedesktop.bolt.xml"
 
 #define BOLT_DBUS_NAME "org.freedesktop.bolt"
 #define BOLT_DBUS_PATH "/org/freedesktop/bolt"
@@ -58,5 +63,20 @@
 #define BOLT_DBUS_DOMAIN_INTERFACE "org.freedesktop.bolt1.Domain"
 #define BOLT_DBUS_POWER_INTERFACE "org.freedesktop.bolt1.Power"
 
+/* sysfs */
+#define BOLT_SYSFS_IOMMU "iommu_dma_protection"
+
+/* environment variables */
+#define BOLT_ENV_DBPATH "BOLT_DBPATH"
+#define BOLT_ENV_RUNTIME_DIRECTORY "RUNTIME_DIRECTORY"
+#define BOLT_ENV_STATE_DIRECTORY "STATE_DIRECTORY"
+
 /* other well known names */
 #define INTEL_WMI_THUNDERBOLT_GUID "86CCFD48-205E-4A77-9C48-2021CBEDE341"
+
+/* helper functions */
+
+char *      bolt_gen_object_path (const char *path_base,
+                                  const char *object_id);
+
+G_END_DECLS
