@@ -20,9 +20,12 @@
 
 #include "config.h"
 
+#include "bolt-error.h"
 #include "bolt-macros.h"
 #include "bolt-str.h"
 
+#include <gio/gio.h>
+
 #include <errno.h>
 #include <string.h>
 
@@ -256,7 +259,8 @@ bolt_strstrip (char *string)
 
 gboolean
 bolt_str_parse_as_int (const char *str,
-                       gint       *ret)
+                       gint       *ret,
+                       GError    **error)
 {
   char *end;
   gint64 val;
@@ -266,24 +270,23 @@ bolt_str_parse_as_int (const char *str,
   errno = 0;
   val = g_ascii_strtoll (str, &end, 0);
 
-  /* conversion errors */
-  if (val == 0 && errno != 0)
-    return FALSE;
-
-  /* check over/underflow */
-  if ((val == G_MAXINT64 || val == G_MININT64) &&
-      errno == ERANGE)
-    return FALSE;
-
-  if (str == end)
+  if (val == 0 && (errno != 0 || str == end))
     {
-      errno = -EINVAL;
+      if (errno == 0)
+        errno = EINVAL;
+
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
+                   "failed to parse '%s' as unsigned integer", str);
       return FALSE;
     }
-
-  if (val > G_MAXINT || val < G_MININT)
+  else if (((val == G_MAXINT64 || val == G_MININT64) && errno == ERANGE) ||
+           val > G_MAXINT || val < G_MININT)
     {
-      errno = -ERANGE;
+      if (errno == 0)
+        errno = ERANGE;
+
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
+                   "parsing '%s' overflows unsigned integer", str);
       return FALSE;
     }
 
@@ -293,6 +296,80 @@ bolt_str_parse_as_int (const char *str,
   return TRUE;
 }
 
+gboolean
+bolt_str_parse_as_uint64 (const char *str,
+                          guint64    *ret,
+                          GError    **error)
+{
+  guint64 val;
+  char *end;
+
+  g_return_val_if_fail (str != NULL, FALSE);
+
+  errno = 0;
+  val = g_ascii_strtoull (str, &end, 0);
+
+  if (val == 0 && (errno != 0 || str == end))
+    {
+      if (errno == 0)
+        errno = EINVAL;
+
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
+                   "failed to parse '%s' as unsigned integer", str);
+      return FALSE;
+    }
+  else if (val == G_MAXUINT64 && errno == ERANGE)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
+                   "parsing '%s' overflows unsigned integer", str);
+      return FALSE;
+    }
+
+  if (ret)
+    *ret = val;
+
+  return TRUE;
+}
+
+gboolean
+bolt_str_parse_as_boolean (const char *str,
+                           gboolean   *ret,
+                           GError    **error)
+{
+  struct
+  {
+    const char *s;
+    gboolean    v;
+  } table[] = {
+    {"1",    TRUE}, {"0",     FALSE},
+    /* true, false */
+    {"t",    TRUE}, {"f",     FALSE},
+    {"true", TRUE}, {"false", FALSE},
+    /* yes, no */
+    {"y",    TRUE}, {"n",     FALSE},
+    {"yes",  TRUE}, {"no",    FALSE},
+    /* on, off */
+    {"on",   TRUE}, {"off",   FALSE},
+  };
+
+  g_return_val_if_fail (str != NULL, FALSE);
+
+  for (gsize i = 0; i < G_N_ELEMENTS (table); i++)
+    {
+      if (bolt_strcaseeq (table[i].s, str))
+        {
+          if (ret)
+            *ret = table[i].v;
+
+          return TRUE;
+        }
+    }
+
+  g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
+               "failed to parse '%s' as boolean", str);
+  return FALSE;
+}
+
 gboolean
 bolt_set_strdup_printf (char      **target,
                         const char *fmt,
