@@ -30,7 +30,8 @@ void bolt_erase_n (void *data,
 void bolt_str_erase (char *str);
 void bolt_str_erase_clear (char **str);
 
-#define bolt_streq(s1, s2) (g_strcmp0 (s1, s2) == 0)
+#define bolt_streq(s1, s2) (g_strcmp0 ((s1), (s2)) == 0)
+#define bolt_strcaseeq(s1, s2) (g_ascii_strcasecmp ((s1), (s2)) == 0)
 
 GStrv    bolt_strv_from_ptr_array (GPtrArray **array);
 gsize    bolt_strv_length (char * const *strv);
@@ -57,7 +58,16 @@ char *bolt_strdup_validate (const char *string);
 char *bolt_strstrip (char *string);
 
 gboolean bolt_str_parse_as_int (const char *str,
-                                gint       *ret);
+                                gint       *ret,
+                                GError    **error);
+
+gboolean bolt_str_parse_as_uint64 (const char *str,
+                                   guint64    *ret,
+                                   GError    **error);
+
+gboolean bolt_str_parse_as_boolean (const char *str,
+                                    gboolean   *ret,
+                                    GError    **error);
 
 /* replacing string pointers, like g_set_object, g_set_error ... */
 static inline gboolean
