@@ -27,4 +27,7 @@ G_BEGIN_DECLS
 
 gboolean     bolt_pid_is_alive (pid_t pid);
 
+gboolean     bolt_sd_notify_literal (const char *state,
+                                     gboolean   *sent,
+                                     GError    **error);
 G_END_DECLS
