@@ -60,7 +60,7 @@ gboolean         mock_sysfs_force_power_enabled (MockSysfs *ms);
 
 const char *     mock_sysfs_domain_add (MockSysfs   *ms,
                                         BoltSecurity security,
-                                        GStrv        bootacl);
+                                        ...) G_GNUC_NULL_TERMINATED;
 
 const char *     mock_sysfs_domain_get_syspath (MockSysfs  *ms,
                                                 const char *id);
@@ -77,6 +77,11 @@ gboolean         mock_sysfs_domain_bootacl_set (MockSysfs  *ms,
                                                 GStrv       acl,
                                                 GError    **error);
 
+gboolean         mock_syfs_domain_iommu_set (MockSysfs  *ms,
+                                             const char *id,
+                                             const char *val,
+                                             GError    **error);
+
 const char *     mock_sysfs_host_add (MockSysfs  *ms,
                                       const char *domain,
                                       MockDevId  *id);
@@ -97,31 +102,4 @@ const char *     mock_sysfs_device_get_parent (MockSysfs  *ms,
 gboolean         mock_sysfs_device_remove (MockSysfs  *ms,
                                            const char *id);
 
-/* helper macro */
-
-/* *INDENT-OFF* */
-#define bolt_assert_strv_equal(a, b, n) G_STMT_START {                                \
-    const GStrv sa__ = (a);                                                           \
-    const GStrv sb__ = (b);                                                           \
-    guint al__ = a != NULL ? g_strv_length (sa__) : 0;                                \
-    guint bl__ = b != NULL ? g_strv_length (sb__) : 0;                                \
-    if (n > 0) {                                                                      \
-        al__ = MIN ((guint) n, al__);                                                 \
-        bl__ = MIN ((guint) n, bl__);                                                 \
-      }                                                                               \
-    if (al__ != bl__)                                                                 \
-      g_assertion_message_cmpnum (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC,        \
-                                  "len(" #a ") == len(" #b ")",                       \
-                                  (long double) al__, "!=", (long double) bl__, 'i'); \
-    else                                                                              \
-      for (guint il__ = 0; il__ < al__; il__++)                                       \
-        if (g_strcmp0 (sa__[il__], sb__[il__]) == 0) {; } else {                      \
-          g_autofree char *va__ = NULL;                                               \
-          va__ = g_strdup_printf (#a "[%u] != " #b "[%u]", il__, il__ );              \
-          g_assertion_message_cmpstr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC,    \
-                                      va__, sa__[il__], "!=", sb__[il__]);            \
-        }                                                                             \
-    } G_STMT_END
-/* *INDENT-ON* */
-
 G_END_DECLS
