@@ -20,6 +20,7 @@
 
 #include "config.h"
 
+#include "bolt-dbus.h"
 #include "bolt-enums.h"
 #include "bolt-error.h"
 #include "bolt-fs.h"
@@ -28,11 +29,13 @@
 #include "bolt-rnd.h"
 #include "bolt-str.h"
 #include "bolt-term.h"
+#include "bolt-test.h"
 #include "bolt-time.h"
 #include "bolt-unix.h"
 #include "mock-sysfs.h"
 
 #include "test-enums.h"
+#include "bolt-test-resources.h"
 
 #include <glib.h>
 #include <gio/gio.h>
@@ -52,7 +55,68 @@
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (GEnumClass, g_type_class_unref);
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (GFlagsClass, g_type_class_unref);
 #endif
+typedef struct
+{
+  int dummy;
+} TestDummy;
 
+#define TEST_DBUS_GRESOURCE_PATH "/bolt/tests/exported/example.bolt.xml"
+#define TEST_DBUS_INTERFACE "org.gnome.bolt.Example"
+
+static void
+test_dbus_interface_info_find (TestDummy *tt, gconstpointer user_data)
+{
+  g_autoptr(GBytes) data = NULL;
+  g_autoptr(GError) error = NULL;
+  GDBusInterfaceInfo *info = NULL;
+  const char *xml;
+
+  data = g_resources_lookup_data (TEST_DBUS_GRESOURCE_PATH,
+                                  G_RESOURCE_LOOKUP_FLAGS_NONE,
+                                  &error);
+  g_assert_no_error (error);
+  g_assert_nonnull (data);
+
+  xml = g_bytes_get_data (data, NULL);
+
+  info = bolt_dbus_interface_info_find (xml, "NON-EXISTENT", &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND);
+  g_assert_null (info);
+  g_clear_error (&error);
+
+  info = bolt_dbus_interface_info_find (xml, TEST_DBUS_INTERFACE, &error);
+  g_assert_no_error (error);
+  g_assert_nonnull (info);
+  g_dbus_interface_info_unref (info);
+}
+
+static void
+test_dbus_interface_info_lookup (TestDummy *tt, gconstpointer user_data)
+{
+  g_autoptr(GError) error = NULL;
+  GDBusInterfaceInfo *info = NULL;
+
+  info = bolt_dbus_interface_info_lookup ("NON-EXISTENT",
+                                          "NON-EXISTENT",
+                                          &error);
+  g_assert_error (error, G_RESOURCE_ERROR, G_RESOURCE_ERROR_NOT_FOUND);
+  g_assert_null (info);
+  g_clear_error (&error);
+
+  info = bolt_dbus_interface_info_lookup (TEST_DBUS_GRESOURCE_PATH,
+                                          "NON-EXISTENT",
+                                          &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND);
+  g_assert_null (info);
+  g_clear_error (&error);
+
+  info = bolt_dbus_interface_info_lookup (TEST_DBUS_GRESOURCE_PATH,
+                                          TEST_DBUS_INTERFACE,
+                                          &error);
+  g_assert_no_error (error);
+  g_assert_nonnull (info);
+  g_dbus_interface_info_unref (info);
+}
 
 typedef struct
 {
@@ -266,6 +330,7 @@ test_error (TestRng *tt, gconstpointer user_data)
 
   ok = bolt_error_propagate_stripped (&target, &source);
   g_assert_error (target, BOLT_ERROR, BOLT_ERROR_BADKEY);
+  g_assert_false (ok);
   g_assert_false (g_dbus_error_is_remote_error (target));
   g_assert_cmpstr (target->message, ==, buserr->message);
 
@@ -759,7 +824,7 @@ test_io_errors (TestIO *tt, gconstpointer user_data)
   g_clear_pointer (&err, g_error_free);
 
   ok = bolt_read_int_at (dirfd (root), "readonly", &iv, &err);
-  g_assert_error (err, G_IO_ERROR, G_IO_ERROR_FAILED);
+  g_assert_error (err, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
   g_assert_false (ok);
   g_clear_pointer (&err, g_error_free);
 
@@ -1249,6 +1314,136 @@ test_str_erase (TestRng *tt, gconstpointer user_data)
     g_assert_cmpint (buf[i], ==, 0);
 }
 
+static void
+test_str_parse_int (TestRng *tt, gconstpointer user_data)
+{
+  struct
+  {
+    const char *str;
+    gint        val;
+    gboolean    error;
+  } table[] = {
+    {"0",                                        0,                FALSE},
+    {"1",                                        1,                FALSE},
+    {"-1",                                      -1,                FALSE},
+#if __SIZEOF_INT__ == 4
+    {"2147483647",                      2147483647,                FALSE}, /* MAX_INT */
+    {"-2147483648",                    -2147483648,                FALSE}, /* MIN_INT */
+    {"2147483648",                               0,                TRUE},  /* MAX_INT + 1 */
+    {"-2147483649",                              0,                TRUE},  /* MIN_INT - 1 */
+#elif __SIZEOF_INT__ == 8
+    {"9223372036854775807",    9223372036854775807,                FALSE}, /* MAX_INT */
+    {"-9223372036854775808",  -9223372036854775808,                FALSE}, /* MIN_INT */
+    {"9223372036854775808",                      0,                TRUE},  /* MAX_INT + 1 */
+    {"-9223372036854775809",                     0,                TRUE},  /* MIN_INT - 1 */
+#else
+    #warning __SIZEOF_INT__ not handled
+#endif
+    {"notanint",                                 0,                TRUE},
+    {"9223372036854775808",                      0,                TRUE}, /* overflow */
+    {"-9223372036854775809",                     0,                TRUE}, /* underflow */
+  };
+
+  for (gsize i = 0; i < G_N_ELEMENTS (table); i++)
+    {
+      g_autoptr(GError) error = NULL;
+      gboolean ok;
+      gint v;
+
+      errno = 0;
+      ok = bolt_str_parse_as_int (table[i].str, &v, &error);
+
+      if (table[i].error)
+        {
+          int err = errno;
+
+          g_assert_nonnull (error);
+          g_assert_false (ok);
+          g_assert_cmpint (err, !=, 0);
+        }
+      else
+        {
+          g_assert_cmpuint (table[i].val, ==, v);
+          g_assert_true (ok);
+        }
+    }
+}
+
+static void
+test_str_parse_uint64 (TestRng *tt, gconstpointer user_data)
+{
+  struct
+  {
+    const char *str;
+    guint64     val;
+    gboolean    error;
+  } table[] = {
+    {"0",                    0,                  FALSE},
+    {"1",                    1,                  FALSE},
+    {"0xffffffffffffffff",   0xffffffffffffffff, FALSE}, /* G_MAXUINT64 */
+    {"notauint64",           0,                  TRUE},
+    {"18446744073709551616", 0,                  TRUE}, /* overflow (G_MAXUINT64 + 1) */
+  };
+
+  for (gsize i = 0; i < G_N_ELEMENTS (table); i++)
+    {
+      g_autoptr(GError) err = NULL;
+      gboolean ok;
+      guint64 v;
+
+      ok = bolt_str_parse_as_uint64 (table[i].str, &v, &err);
+      if (table[i].error)
+        {
+          g_assert_nonnull (err);
+          g_assert_false (ok);
+        }
+      else
+        {
+          g_assert_cmpuint (table[i].val, ==, v);
+          g_assert_true (ok);
+        }
+    }
+}
+
+static void
+test_str_parse_boolean (TestRng *tt, gconstpointer user_data)
+{
+  struct
+  {
+    const char *str;
+    gboolean    val;
+    gboolean    error;
+  } table[] = {
+    {"TRUE",      TRUE,  FALSE},
+    {"YES",       TRUE,  FALSE},
+    {"1",         TRUE,  FALSE},
+    {"FALSE",     FALSE, FALSE},
+    {"no",        FALSE, FALSE},
+    {"0",         FALSE, FALSE},
+    {"notabool",  FALSE, TRUE},
+    {"12",        FALSE, TRUE},
+  };
+
+  for (gsize i = 0; i < G_N_ELEMENTS (table); i++)
+    {
+      g_autoptr(GError) err = NULL;
+      gboolean ok;
+      gboolean v;
+
+      ok = bolt_str_parse_as_boolean (table[i].str, &v, &err);
+      if (table[i].error)
+        {
+          g_assert_nonnull (err);
+          g_assert_false (ok);
+        }
+      else
+        {
+          g_assert_cmpuint (table[i].val, ==, v);
+          g_assert_true (ok);
+        }
+    }
+}
+
 static void
 test_str_set (TestRng *tt, gconstpointer user_data)
 {
@@ -1685,6 +1880,22 @@ main (int argc, char **argv)
 
   g_test_init (&argc, &argv, NULL);
 
+  g_resources_register (bolt_test_get_resource ());
+
+  g_test_add ("/common/dbus/interface_info_find",
+              TestDummy,
+              NULL,
+              NULL,
+              test_dbus_interface_info_find,
+              NULL);
+
+  g_test_add ("/common/dbus/interface_info_lookup",
+              TestDummy,
+              NULL,
+              NULL,
+              test_dbus_interface_info_lookup,
+              NULL);
+
   g_test_add ("/common/enums",
               TestRng,
               NULL,
@@ -1776,6 +1987,27 @@ main (int argc, char **argv)
               test_str_erase,
               NULL);
 
+  g_test_add ("/common/str/parse/int",
+              TestRng,
+              NULL,
+              NULL,
+              test_str_parse_int,
+              NULL);
+
+  g_test_add ("/common/str/parse/uint64",
+              TestRng,
+              NULL,
+              NULL,
+              test_str_parse_uint64,
+              NULL);
+
+  g_test_add ("/common/str/parse/boolean",
+              TestRng,
+              NULL,
+              NULL,
+              test_str_parse_boolean,
+              NULL);
+
   g_test_add ("/common/str/set",
               TestRng,
               NULL,
