@@ -22,12 +22,11 @@
 
 #include "bolt-power.h"
 
+#include "bolt-dbus.h"
 #include "bolt-fs.h"
 #include "bolt-str.h"
 #include "mock-sysfs.h"
 
-#include "bolt-daemon-resource.h"
-
 #include <glib.h>
 #include <gio/gio.h>
 #include <glib/gprintf.h>
@@ -139,7 +138,7 @@ test_power_basic (TestPower *tt, gconstpointer user)
                 "timeout", &timeout,
                 NULL);
 
-  g_assert_nonnull (rundir);
+  g_assert_cmpstr (rundir, ==, tt->rundir);
   g_assert_nonnull (statedir);
   g_assert_nonnull (udev);
   g_assert (udev == tt->udev);
@@ -734,7 +733,7 @@ main (int argc, char **argv)
 
   g_test_init (&argc, &argv, NULL);
 
-  g_resources_register (bolt_daemon_get_resource ());
+  bolt_dbus_ensure_resources ();
 
   g_test_add ("/power/basic",
               TestPower,
