@@ -20,15 +20,16 @@
 
 #include "config.h"
 
+#include "bolt-dbus.h"
 #include "bolt-error.h"
 #include "bolt-fs.h"
 #include "bolt-io.h"
 #include "bolt-str.h"
+#include "bolt-test.h"
 
 #include "bolt-config.h"
 #include "bolt-store.h"
 
-#include "bolt-daemon-resource.h"
 #include "mock-sysfs.h"
 
 #include <glib.h>
@@ -160,7 +161,7 @@ test_store_basic (TestStore *tt, gconstpointer user_data)
   g_clear_error (&error);
   g_assert_no_error (error);
 
-  key = bolt_key_new ();
+  key = bolt_key_new (NULL);
   g_assert_nonnull (key);
 
   ok = bolt_store_put_device (tt->store, dev, BOLT_POLICY_MANUAL, key, &error);
@@ -316,7 +317,7 @@ test_key (TestStore *tt, gconstpointer user_data)
   guint32 mode;
   int r;
 
-  key = bolt_key_new ();
+  key = bolt_key_new (NULL);
   g_assert_nonnull (key);
 
   g_object_get (key, "fresh", &fresh, NULL);
@@ -681,7 +682,7 @@ main (int argc, char **argv)
 
   g_test_init (&argc, &argv, NULL);
 
-  g_resources_register (bolt_daemon_get_resource ());
+  bolt_dbus_ensure_resources ();
 
   g_test_add ("/daemon/key",
               TestStore,
