@@ -20,6 +20,7 @@
 
 #include "config.h"
 
+#include "bolt-dbus.h"
 #include "bolt-macros.h"
 #include "bolt-store.h"
 #include "bolt-str.h"
@@ -29,8 +30,6 @@
 #include "bolt-test.h"
 #include "mock-sysfs.h"
 
-#include "bolt-daemon-resource.h"
-
 #include <glib.h>
 #include <gio/gio.h>
 #include <glib/gprintf.h>
@@ -145,6 +144,97 @@ test_sysfs_domain_for_device (TestSysfs *tt, gconstpointer user)
   mock_sysfs_domain_remove (tt->sysfs, domain);
 }
 
+static void
+test_sysfs_read_iommu (TestSysfs *tt, gconstpointer user)
+{
+  const char *domain;
+  const char *syspath;
+  gboolean ok;
+
+  domain = mock_sysfs_domain_add (tt->sysfs, BOLT_SECURITY_SECURE, NULL);
+  g_assert_nonnull (domain);
+
+  syspath = mock_sysfs_domain_get_syspath (tt->sysfs, domain);
+  g_assert_nonnull (syspath);
+
+  /* no sysfs attribute at all */
+  {
+    g_autoptr(GError) err = NULL;
+    g_autoptr(udev_device) udev = NULL;
+    gboolean iommu;
+
+    udev = udev_device_new_from_syspath (tt->udev, syspath);
+    g_assert_nonnull (udev);
+
+    iommu = TRUE; /* we expect FALSE */
+    ok = bolt_sysfs_read_iommu (udev, &iommu, &err);
+    g_assert_no_error (err);
+    g_assert_true (ok);
+    g_assert_false (iommu);
+  }
+
+  /* sysfs attribute is "0" */
+  {
+    g_autoptr(GError) err = NULL;
+    g_autoptr(udev_device) udev = NULL;
+    gboolean iommu;
+
+    ok = mock_syfs_domain_iommu_set (tt->sysfs, domain, "0", &err);
+    g_assert_no_error (err);
+    g_assert_true (ok);
+
+    udev = udev_device_new_from_syspath (tt->udev, syspath);
+    g_assert_nonnull (udev);
+
+    iommu = TRUE; /* we expect FALSE */
+    ok = bolt_sysfs_read_iommu (udev, &iommu, &err);
+    g_assert_no_error (err);
+    g_assert_true (ok);
+    g_assert_false (iommu);
+  }
+
+  /* sysfs attribute is "1" */
+  {
+    g_autoptr(GError) err = NULL;
+    g_autoptr(udev_device) udev = NULL;
+    gboolean iommu;
+
+    ok = mock_syfs_domain_iommu_set (tt->sysfs, domain, "1", &err);
+    g_assert_no_error (err);
+    g_assert_true (ok);
+
+    udev = udev_device_new_from_syspath (tt->udev, syspath);
+    g_assert_nonnull (udev);
+
+    iommu = FALSE; /* now we expect TRUE */
+    ok = bolt_sysfs_read_iommu (udev, &iommu, &err);
+    g_assert_no_error (err);
+    g_assert_true (ok);
+    g_assert_true (iommu);
+  }
+
+  /* sysfs attribute contains garbage */
+  {
+    g_autoptr(GError) err = NULL;
+    g_autoptr(udev_device) udev = NULL;
+    gboolean iommu;
+
+    ok = mock_syfs_domain_iommu_set (tt->sysfs, domain, "garbage", &err);
+    g_assert_no_error (err);
+    g_assert_true (ok);
+
+    udev = udev_device_new_from_syspath (tt->udev, syspath);
+    g_assert_nonnull (udev);
+
+    iommu = TRUE; /* should be unchanged */
+    ok = bolt_sysfs_read_iommu (udev, &iommu, &err);
+    g_assert_nonnull (err);
+    g_assert_false (ok);
+    g_assert_true (iommu);
+  }
+
+}
+
 static void
 test_sysfs_domains (TestSysfs *tt, gconstpointer user)
 {
@@ -254,6 +344,50 @@ test_sysfs_domains (TestSysfs *tt, gconstpointer user)
     g_assert_null (all[i]);
 }
 
+static void
+test_sysfs_domain_connect (TestSysfs *tt, gconstpointer user)
+{
+  g_autoptr(udev_device) udevice = NULL;
+  g_autoptr(BoltDomain) domain = NULL;
+  g_autoptr(GError) err = NULL;
+  const char *uid = "884c6edd-7118-4b21-b186-b02d396ecca0";
+  const char *id;
+  const char *syspath;
+  BoltSecurity security;
+
+  domain = g_object_new (BOLT_TYPE_DOMAIN,
+                         "store", NULL,
+                         "uid", uid,
+                         "bootacl", NULL,
+                         NULL);
+
+  g_assert_nonnull (domain);
+  g_assert_false (bolt_domain_has_iommu (domain));
+  security = bolt_domain_get_security (domain);
+  g_assert_cmpint (security, ==, BOLT_SECURITY_UNKNOWN);
+  g_assert_null (bolt_domain_get_syspath (domain));
+
+  id = mock_sysfs_domain_add (tt->sysfs,
+                              BOLT_SECURITY_SECURE,
+                              "iommu", "1\n",
+                              NULL);
+
+  syspath = mock_sysfs_domain_get_syspath (tt->sysfs, id);
+  udevice = udev_device_new_from_syspath (tt->udev, syspath);
+
+  g_assert_nonnull (udevice);
+
+  bolt_domain_connected (domain, udevice);
+  security = bolt_domain_get_security (domain);
+  g_assert_cmpint (security, ==, BOLT_SECURITY_SECURE);
+
+  g_assert_cmpstr (syspath,
+                   ==,
+                   bolt_domain_get_syspath (domain));
+
+  g_assert_true (bolt_domain_has_iommu (domain));
+}
+
 typedef struct
 {
   MockSysfs   *sysfs;
@@ -287,7 +421,8 @@ test_bootacl_setup (TestBootacl *tt, gconstpointer data)
   str = g_strnfill (tt->slots - 1, ',');
   tt->acl = g_strsplit (str, ",", 1024);
 
-  tt->dom_sysid = mock_sysfs_domain_add (tt->sysfs, BOLT_SECURITY_USER, tt->acl);
+  tt->dom_sysid = mock_sysfs_domain_add (tt->sysfs, BOLT_SECURITY_USER,
+                                         "bootacl", tt->acl, NULL);
 
   syspath = mock_sysfs_domain_get_syspath (tt->sysfs, tt->dom_sysid);
   udevice = udev_device_new_from_syspath (tt->udev, syspath);
@@ -706,6 +841,7 @@ test_bootacl_update_udev (TestBootacl *tt, gconstpointer user)
 
       used = bolt_domain_bootacl_get_used (dom, &n_used);
       g_assert_cmpuint (n_used, ==, i + 1);
+      g_assert_nonnull (used);
 
       have = bolt_domain_get_bootacl (dom);
       bolt_assert_strv_equal (have, acl, -1);
@@ -973,7 +1109,7 @@ main (int argc, char **argv)
 
   g_test_init (&argc, &argv, NULL);
 
-  g_resources_register (bolt_daemon_get_resource ());
+  bolt_dbus_ensure_resources ();
 
   g_test_add ("/sysfs/domain_for_device",
               TestSysfs,
@@ -982,6 +1118,13 @@ main (int argc, char **argv)
               test_sysfs_domain_for_device,
               test_sysfs_tear_down);
 
+  g_test_add ("/sysfs/read_iommu",
+              TestSysfs,
+              NULL,
+              test_sysfs_setup,
+              test_sysfs_read_iommu,
+              test_sysfs_tear_down);
+
   g_test_add ("/sysfs/domain/basic",
               TestSysfs,
               NULL,
@@ -989,6 +1132,13 @@ main (int argc, char **argv)
               test_sysfs_domains,
               test_sysfs_tear_down);
 
+  g_test_add ("/sysfs/domain/connect",
+              TestSysfs,
+              NULL,
+              test_sysfs_setup,
+              test_sysfs_domain_connect,
+              test_sysfs_tear_down);
+
   g_test_add ("/sysfs/domain/bootacl/basic",
               TestBootacl,
               NULL,
